//----------------------------------------------------------------------------
// user_logic.v - module
//----------------------------------------------------------------------------
//
// ***************************************************************************
// ** Copyright (c) 1995-2006 Xilinx, Inc.  All rights reserved.            **
// **                                                                       **
// ** Xilinx, Inc.                                                          **
// ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
// ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
// ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
// ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
// ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
// ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
// ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
// ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
// ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
// ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
// ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
// ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
// ** FOR A PARTICULAR PURPOSE.                                             **
// **                                                                       **
// ***************************************************************************
//
//----------------------------------------------------------------------------
// Filename:          user_logic.v
// Version:           1.01.a
// Description:       User logic module.
// Date:              Thu Mar 20 17:24:23 2008 (by Create and Import Peripheral Wizard)
// Verilog Standard:  Verilog-2001
//----------------------------------------------------------------------------
// Naming Conventions:
//   active low signals:                    "*_n"
//   clock signals:                         "clk", "clk_div#", "clk_#x"
//   reset signals:                         "rst", "rst_n"
//   generics:                              "C_*"
//   user defined types:                    "*_TYPE"
//   state machine next state:              "*_ns"
//   state machine current state:           "*_cs"
//   combinatorial signals:                 "*_com"
//   pipelined or register delay signals:   "*_d#"
//   counter signals:                       "*cnt*"
//   clock enable signals:                  "*_ce"
//   internal version of output port:       "*_i"
//   device pins:                           "*_pin"
//   ports:                                 "- Names begin with Uppercase"
//   processes:                             "*_PROCESS"
//   component instantiations:              "<ENTITY_>I_<#|FUNC>"
//----------------------------------------------------------------------------

module user_logic
(
  // -- ADD USER PORTS BELOW THIS LINE ---------------
  // --USER ports added here 
  // -- ADD USER PORTS ABOVE THIS LINE ---------------

  // -- DO NOT EDIT BELOW THIS LINE ------------------
  // -- Bus protocol ports, do not add to or delete 
  Bus2IP_Clk,                     // Bus to IP clock
  Bus2IP_Reset,                   // Bus to IP reset
  IP2Bus_IntrEvent,               // IP to Bus interrupt event
  Bus2IP_Data,                    // Bus to IP data bus for user logic
  Bus2IP_BE,                      // Bus to IP byte enables for user logic
  Bus2IP_Burst,                   // Bus to IP burst-mode qualifier
  Bus2IP_RdCE,                    // Bus to IP read chip enable for user logic
  Bus2IP_WrCE,                    // Bus to IP write chip enable for user logic
  Bus2IP_RdReq,                   // Bus to IP read request
  Bus2IP_WrReq,                   // Bus to IP write request
  IP2Bus_Data,                    // IP to Bus data bus for user logic
  IP2Bus_Retry,                   // IP to Bus retry response
  IP2Bus_Error,                   // IP to Bus error response
  IP2Bus_ToutSup,                 // IP to Bus timeout suppress
  IP2Bus_RdAck,                   // IP to Bus read transfer acknowledgement
  IP2Bus_WrAck,                   // IP to Bus write transfer acknowledgement
  Bus2IP_MstError,                // Bus to IP master error
  Bus2IP_MstLastAck,              // Bus to IP master last acknowledge
  Bus2IP_MstRdAck,                // Bus to IP master read acknowledge
  Bus2IP_MstWrAck,                // Bus to IP master write acknowledge
  Bus2IP_MstRetry,                // Bus to IP master retry
  Bus2IP_MstTimeOut,              // Bus to IP mster timeout
  IP2Bus_Addr,                    // IP to Bus address for the master transaction
  IP2Bus_MstBE,                   // IP to Bus byte-enables qualifiers
  IP2Bus_MstBurst,                // IP to Bus burst qualifier
  IP2Bus_MstBusLock,              // IP to Bus bus-lock qualifier
  IP2Bus_MstNum,                  // IP to Bus burst size indicator
  IP2Bus_MstRdReq,                // IP to Bus master read request
  IP2Bus_MstWrReq,                // IP to Bus master write request
  IP2IP_Addr,                     // IP to IP local device address for the master transaction
  IP2RFIFO_WrReq,                 // IP to RFIFO : IP write request
  IP2RFIFO_Data,                  // IP to RFIFO : IP write data
  RFIFO2IP_WrAck,                 // RFIFO to IP : RFIFO write acknowledge
  RFIFO2IP_AlmostFull,            // RFIFO to IP : RFIFO almost full
  RFIFO2IP_Full,                  // RFIFO to IP : RFIFO full
  IP2WFIFO_RdReq,                 // IP to WFIFO : IP read request
  WFIFO2IP_Data,                  // WFIFO to IP : WFIFO read data
  WFIFO2IP_RdAck,                 // WFIFO to IP : WFIFO read acknowledge
  WFIFO2IP_AlmostEmpty,           // WFIFO to IP : WFIFO almost empty
  WFIFO2IP_Empty                  // WFIFO to IP : WFIFO empty
  // -- DO NOT EDIT ABOVE THIS LINE ------------------
); // user_logic

// -- ADD USER PARAMETERS BELOW THIS LINE ------------
// --USER parameters added here 
// -- ADD USER PARAMETERS ABOVE THIS LINE ------------

// -- DO NOT EDIT BELOW THIS LINE --------------------
// -- Bus protocol parameters, do not add to or delete
parameter C_AWIDTH                       = 32;
parameter C_DWIDTH                       = 64;
parameter C_NUM_CE                       = 18;
parameter C_IP_INTR_NUM                  = 1;
parameter C_RDFIFO_DWIDTH                = 64;
parameter C_WRFIFO_DWIDTH                = 64;
// -- DO NOT EDIT ABOVE THIS LINE --------------------

// -- ADD USER PORTS BELOW THIS LINE -----------------
// --USER ports added here 
// -- ADD USER PORTS ABOVE THIS LINE -----------------

// -- DO NOT EDIT BELOW THIS LINE --------------------
// -- Bus protocol ports, do not add to or delete
input                                     Bus2IP_Clk;
input                                     Bus2IP_Reset;
output     [0 : C_IP_INTR_NUM-1]          IP2Bus_IntrEvent;
input      [0 : C_DWIDTH-1]               Bus2IP_Data;
input      [0 : C_DWIDTH/8-1]             Bus2IP_BE;
input                                     Bus2IP_Burst;
input      [0 : C_NUM_CE-1]               Bus2IP_RdCE;
input      [0 : C_NUM_CE-1]               Bus2IP_WrCE;
input                                     Bus2IP_RdReq;
input                                     Bus2IP_WrReq;
output     [0 : C_DWIDTH-1]               IP2Bus_Data;
output                                    IP2Bus_Retry;
output                                    IP2Bus_Error;
output                                    IP2Bus_ToutSup;
output                                    IP2Bus_RdAck;
output                                    IP2Bus_WrAck;
input                                     Bus2IP_MstError;
input                                     Bus2IP_MstLastAck;
input                                     Bus2IP_MstRdAck;
input                                     Bus2IP_MstWrAck;
input                                     Bus2IP_MstRetry;
input                                     Bus2IP_MstTimeOut;
output     [0 : C_AWIDTH-1]               IP2Bus_Addr;
output     [0 : C_DWIDTH/8-1]             IP2Bus_MstBE;
output                                    IP2Bus_MstBurst;
output                                    IP2Bus_MstBusLock;
output     [0 : 4]                        IP2Bus_MstNum;
output                                    IP2Bus_MstRdReq;
output                                    IP2Bus_MstWrReq;
output     [0 : C_AWIDTH-1]               IP2IP_Addr;
output                                    IP2RFIFO_WrReq;
output     [0 : C_RDFIFO_DWIDTH-1]        IP2RFIFO_Data;
input                                     RFIFO2IP_WrAck;
input                                     RFIFO2IP_AlmostFull;
input                                     RFIFO2IP_Full;
output                                    IP2WFIFO_RdReq;
input      [0 : C_WRFIFO_DWIDTH-1]        WFIFO2IP_Data;
input                                     WFIFO2IP_RdAck;
input                                     WFIFO2IP_AlmostEmpty;
input                                     WFIFO2IP_Empty;
// -- DO NOT EDIT ABOVE THIS LINE --------------------

//----------------------------------------------------------------------------
// Implementation
//----------------------------------------------------------------------------

  // --USER nets declarations added here, as needed for user logic

  // Nets for user logic slave model s/w accessible register example
  reg        [0 : C_DWIDTH-1]               slv_reg0;
  reg        [0 : C_DWIDTH-1]               slv_reg1	= 'h00010001;
  reg        [0 : C_DWIDTH-1]               slv_reg2    = 'h00020002;
  reg        [0 : C_DWIDTH-1]               slv_reg3    = 'h00030003;
  reg        [0 : C_DWIDTH-1]               slv_reg4    = 'h00040004;
  reg        [0 : C_DWIDTH-1]               slv_reg5    = 'h00050005;
  reg        [0 : C_DWIDTH-1]               slv_reg6    = 'h00060006;
  reg        [0 : C_DWIDTH-1]               slv_reg7    = 'h00070007;
  reg        [0 : C_DWIDTH-1]               slv_reg8    = 'h00080008;
  reg        [0 : C_DWIDTH-1]               slv_reg9    = 'h00090009;
  reg        [0 : C_DWIDTH-1]               slv_reg10   = 'h000A000A;
  reg        [0 : C_DWIDTH-1]               slv_reg11   = 'h000B000B;
  reg        [0 : C_DWIDTH-1]               slv_reg12   = 'h000C000C;
  reg        [0 : C_DWIDTH-1]               slv_reg13   = 'h000D000D;
  reg        [0 : C_DWIDTH-1]               slv_reg14   = 'h000E000E;
  reg        [0 : C_DWIDTH-1]               slv_reg15   = 'h000F000F;
  wire       [0 : 15]                       slv_reg_write_select;
  wire       [0 : 15]                       slv_reg_read_select;
  reg        [0 : C_DWIDTH-1]               slv_ip2bus_data;
  wire                                      slv_read_ack;
  wire                                      slv_write_ack;
  integer                                   byte_index, bit_index;

  // --USER logic implementation added here
reg [5:0] state = 0;
reg done = 0;


always@( posedge Bus2IP_Clk ) begin
	case( state )
		0: begin
			if( (slv_reg0 == 'hDEAD_BEEF) && (done == 0)  )
				state = 1;
			if( slv_reg0 == 'hFEED_BEEF )		//operate once, but able to reset
				done = 0;
		end
		23: begin
			slv_reg1 = Bus2IP_Data [32:63];
			slv_reg2 = Bus2IP_Data [0:31];
			state = 24;
			 end
		24: begin
			slv_reg3 = Bus2IP_Data [32:63];
			slv_reg4 = Bus2IP_Data [0:31];
			state = 25;
			end
		25: begin
			slv_reg5 = Bus2IP_Data [32:63];
			slv_reg6 = Bus2IP_Data [0:31];
			state = 26;
			 end
		26: begin
			slv_reg7 = Bus2IP_Data [32:63];
			slv_reg8 = Bus2IP_Data [0:31];
			state = 27;
			 end
		27: begin
			slv_reg9 = Bus2IP_Data [32:63];
			slv_reg10 = Bus2IP_Data [0:31];
			state = 28;
			 end
		28: begin
			slv_reg11 = Bus2IP_Data [32:63];
			slv_reg12 = Bus2IP_Data [0:31];
			state = 29;
			 end
		29: begin
			slv_reg13 = Bus2IP_Data [32:63];
			slv_reg14 = Bus2IP_Data [0:31];			
			state = 30;
			 end
		31: begin
			state = 0;
			done = 1;
			end
		default:
			state = state + 1;	
	endcase
end

assign	IP2Bus_MstRdReq = ( state != 0 );
assign	IP2Bus_MstWrReq = 'b0;
assign	IP2Bus_MstBurst = ( state != 0 );
assign	IP2Bus_MstBusLock = 'b0;
assign	IP2Bus_MstNum [0:4] = 'b01000;
assign	IP2Bus_Addr [0:31] = 0;				//read address zero
assign	IP2Bus_MstBE [0:7] = 'b1111_1111;
assign	IP2IP_Addr [0:31] = 0;				//write address zero (dunno if it's valid!)
//assign	IP2Bus_WrAck = ( state < 30 ) && ( state > 22 );


  // ------------------------------------------------------
  // Example code to read/write user logic slave model s/w accessible registers
  // 
  // Note:
  // The example code presented here is to show you one way of reading/writing
  // software accessible registers implemented in the user logic slave model.
  // Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
  // to one software accessible register by the top level template. For example,
  // if you have four 32 bit software accessible registers in the user logic, you
  // are basically operating on the following memory mapped registers:
  // 
  //    Bus2IP_WrCE or   Memory Mapped
  //       Bus2IP_RdCE   Register
  //            "1000"   C_BASEADDR + 0x0
  //            "0100"   C_BASEADDR + 0x4
  //            "0010"   C_BASEADDR + 0x8
  //            "0001"   C_BASEADDR + 0xC
  // 
  // ------------------------------------------------------
  
  assign
    slv_reg_write_select = Bus2IP_WrCE[0:15],
    slv_reg_read_select  = Bus2IP_RdCE[0:15],
    slv_write_ack        = Bus2IP_WrCE[0] || Bus2IP_WrCE[1] || Bus2IP_WrCE[2] || Bus2IP_WrCE[3] || Bus2IP_WrCE[4] || Bus2IP_WrCE[5] || Bus2IP_WrCE[6] || Bus2IP_WrCE[7] || Bus2IP_WrCE[8] || Bus2IP_WrCE[9] || Bus2IP_WrCE[10] || Bus2IP_WrCE[11] || Bus2IP_WrCE[12] || Bus2IP_WrCE[13] || Bus2IP_WrCE[14] || Bus2IP_WrCE[15],
    slv_read_ack         = Bus2IP_RdCE[0] || Bus2IP_RdCE[1] || Bus2IP_RdCE[2] || Bus2IP_RdCE[3] || Bus2IP_RdCE[4] || Bus2IP_RdCE[5] || Bus2IP_RdCE[6] || Bus2IP_RdCE[7] || Bus2IP_RdCE[8] || Bus2IP_RdCE[9] || Bus2IP_RdCE[10] || Bus2IP_RdCE[11] || Bus2IP_RdCE[12] || Bus2IP_RdCE[13] || Bus2IP_RdCE[14] || Bus2IP_RdCE[15];

  // implement slave model register(s)
  always @( posedge Bus2IP_Clk )
    begin: SLAVE_REG_WRITE_PROC

      if ( Bus2IP_Reset == 1 )
        begin
          slv_reg0 <= 0; /*
          slv_reg1 <= 0;
          slv_reg2 <= 0;
          slv_reg3 <= 0;
          slv_reg4 <= 0;
          slv_reg5 <= 0;
          slv_reg6 <= 0;
          slv_reg7 <= 0;
          slv_reg8 <= 0;
          slv_reg9 <= 0;
          slv_reg10 <= 0;
          slv_reg11 <= 0;
          slv_reg12 <= 0;
          slv_reg13 <= 0;
          slv_reg14 <= 0;
          slv_reg15 <= 0; */
        end
      else
        case ( slv_reg_write_select )
          16'b1000000000000000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg0[bit_index] <= Bus2IP_Data[bit_index];
          /*
		  16'b0100000000000000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg1[bit_index] <= Bus2IP_Data[bit_index];
          16'b0010000000000000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg2[bit_index] <= Bus2IP_Data[bit_index];
          16'b0001000000000000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg3[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000100000000000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg4[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000010000000000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg5[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000001000000000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg6[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000000100000000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg7[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000000010000000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg8[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000000001000000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg9[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000000000100000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg10[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000000000010000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg11[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000000000001000 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg12[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000000000000100 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg13[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000000000000010 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg14[bit_index] <= Bus2IP_Data[bit_index];
          16'b0000000000000001 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg15[bit_index] <= Bus2IP_Data[bit_index];
		*/
          default : ;
        endcase

    end // SLAVE_REG_WRITE_PROC

  // implement slave model register read mux
  always @( slv_reg_read_select or slv_reg0 or slv_reg1 or slv_reg2 or slv_reg3 or slv_reg4 or slv_reg5 or slv_reg6 or slv_reg7 or slv_reg8 or slv_reg9 or slv_reg10 or slv_reg11 or slv_reg12 or slv_reg13 or slv_reg14 or slv_reg15 )
    begin: SLAVE_REG_READ_PROC

      case ( slv_reg_read_select )
        16'b1000000000000000 : slv_ip2bus_data <= slv_reg0;
        16'b0100000000000000 : slv_ip2bus_data <= slv_reg1;
        16'b0010000000000000 : slv_ip2bus_data <= slv_reg2;
        16'b0001000000000000 : slv_ip2bus_data <= slv_reg3;
        16'b0000100000000000 : slv_ip2bus_data <= slv_reg4;
        16'b0000010000000000 : slv_ip2bus_data <= slv_reg5;
        16'b0000001000000000 : slv_ip2bus_data <= slv_reg6;
        16'b0000000100000000 : slv_ip2bus_data <= slv_reg7;
        16'b0000000010000000 : slv_ip2bus_data <= slv_reg8;
        16'b0000000001000000 : slv_ip2bus_data <= slv_reg9;
        16'b0000000000100000 : slv_ip2bus_data <= slv_reg10;
        16'b0000000000010000 : slv_ip2bus_data <= slv_reg11;
        16'b0000000000001000 : slv_ip2bus_data <= slv_reg12;
        16'b0000000000000100 : slv_ip2bus_data <= slv_reg13;
        16'b0000000000000010 : slv_ip2bus_data <= slv_reg14;
        16'b0000000000000001 : slv_ip2bus_data <= slv_reg15;
        default : slv_ip2bus_data <= 0;
      endcase

    end // SLAVE_REG_READ_PROC

  // ------------------------------------------------------------
  // Example code to drive IP to Bus signals
  // ------------------------------------------------------------

  assign IP2Bus_Data        = slv_ip2bus_data;
  assign IP2Bus_WrAck       = slv_write_ack || Bus2IP_WrReq; //for reading bursts
  assign IP2Bus_RdAck       = slv_read_ack;
  assign IP2Bus_Error       = 0;
  assign IP2Bus_Retry       = 0;
  assign IP2Bus_ToutSup     = 0;

endmodule
