`timescale 1 ns / 100 ps
module plb_m (
  SYS_plbClk,
  SYS_plbReset,

  PLB_MnAddrAck,
  PLB_MnRearbitrate,
  PLB_Mnssize,
  PLB_MnBusy,
  PLB_MnErr,
  PLB_pendReq,
  PLB_pendPri,
  PLB_reqPri,
  Mn_request,
  Mn_priority,
  Mn_busLock,
  Mn_RNW,
  Mn_BE,
  Mn_size,
  Mn_type,
  Mn_msize,

  Mn_compress,
  Mn_guarded,
  Mn_lockErr,
  Mn_ordered,
  Mn_abort,
  Mn_ABus,

  PLB_MnWrDAck,
  PLB_MnWrBTerm,
  Mn_wrBurst,  
  Mn_wrDBus,
  
  PLB_MnRdDAck, 
  PLB_MnRdBTerm,
  PLB_MnRdWdAddr,
  PLB_MnRdDBus,
  Mn_rdBurst
  );

  input         SYS_plbClk;
  input         SYS_plbReset;

  input         PLB_MnAddrAck;
  input         PLB_MnRearbitrate;
  input  [0:1]  PLB_Mnssize;
  input         PLB_MnBusy;
  input         PLB_MnErr;
  input         PLB_pendReq;
  input  [0:1]  PLB_pendPri;
  input  [0:1]  PLB_reqPri;     
  output        Mn_request;
  output [0:1]  Mn_priority;
  output        Mn_busLock; 
  output        Mn_RNW;
  output [0:7]  Mn_BE;
  output [0:3]  Mn_size;
  output [0:2]  Mn_type;
  output [0:1]  Mn_msize;
  output        Mn_compress;
  output        Mn_guarded;
  output        Mn_ordered;
  output        Mn_lockErr;
  output        Mn_abort;
  output [0:31] Mn_ABus;

  input         PLB_MnRdDAck;
  input         PLB_MnRdBTerm;
  input [0:3]   PLB_MnRdWdAddr;
  input [0:63]  PLB_MnRdDBus; 
  output        Mn_rdBurst;
  
  input         PLB_MnWrDAck;
  input         PLB_MnWrBTerm;
  output        Mn_wrBurst;
  output [0:63] Mn_wrDBus;

   /////////////////////////////////////////////////////////////////

   parameter                  C_CSIZE  = 4;
   parameter                  C_WABUS  = 32'h0000_0000;
   parameter                  C_WDBUS  = 32'hdead_beef;
   parameter                  C_RABUS  = 32'h0000_0040;
   parameter                  C_IABUS  = 32'h0000_0050;

   parameter      [3: 0]      S_IDLE   = 'd0,
                              S_WCOUNT = 'd1,
                              S_WSTART = 'd2,
                              S_WACK   = 'd3,
                              S_WCOMP  = 'd4,
                              S_RCOUNT = 'd5,
                              S_RSTART = 'd6,
                              S_RACK   = 'd7,
                              S_RCOMP  = 'd8,
                              S_ICOUNT = 'd9,
                              S_ISTART = 'd10,
                              S_IACK   = 'd11,
                              S_ICOMP  = 'd12;

   /////////////////////////////////////////////////////////////////

   wire                 clk, rst;

   reg                  rCS, rNS;
   reg   [C_CSIZE: 0]   rCounter;
   reg                  rRequest;
   reg                  rRNW;
   reg   [0: 63]        rABus;
   reg   [0: 63]        rWrDBus;
   reg   [0: 63]        rStore;

   /////////////////////////////////////////////////////////////////

   assign      clk = SYS_plbClk;
   assign      rst = SYS_plbReset;

   assign      Mn_priority = 2'b11;       //Highest (pg31)
   assign      Mn_busLock = 1'b0;         //pg31
   assign      Mn_BE = 8'hFF;             //64-bit (pg38)
   assign      Mn_size = 4'b000;          //transfer one to eight bytes of a word  at the target address (pg45)
   assign      Mn_type = 3'b000;          //pg46
   assign      Mn_msize = 2'b01;          //pg47
   assign      Mn_compress = 1'b0;
   assign      Mn_guarded  = 1'b0;
   assign      Mn_ordered  = 1'b0;
   assign      Mn_lockErr  = 1'b0;
   assign      Mn_abort = 1'b0;

   assign      Mn_rdBurst = 1'b0;
   assign      Mn_wrBurst = 1'b0;

   assign      Mn_request = rRequest;
   assign      Mn_RNW = rRNW;
   assign      Mn_ABus = rABus;
   assign      Mn_wrDBus = rWrDBus;

   always @ (*)
      begin
         case (rCS)
            S_IDLE   :  begin
                           rNS = S_WCOUNT;
                        end
            S_WCOUNT :  begin
                           if (rCounter == {(C_CSIZE){1'b1}})
                              begin
                                 rNS = S_WSTART;
                              end
                           else
                              begin
                                 rNS = S_WCOUNT;
                              end
                        end
            S_WSTART :  begin
                           if (PLB_MnAddrAck)
                              begin
                                 rNS = S_WACK;
                              end
                           else
                              begin
                                 rNS = S_WSTART;
                              end
                        end
            S_WACK   :  begin
                           rNS = S_WCOMP;
                        end
            S_WCOMP  :  begin
                           if (PLB_MnWrDAck)
                              begin
                                 rNS = S_RCOUNT;
                              end
                           else
                              begin
                                 rNS = S_WCOMP;
                              end
                        end
            S_RCOUNT :  begin
                           if (rCounter == {(C_CSIZE){1'b1}})
                              begin
                                 rNS = S_RSTART;
                              end
                           else
                              begin
                                 rNS = S_RCOUNT;
                              end
                        end
            S_RSTART :  begin
                           if (PLB_MnAddrAck)
                              begin
                                 rNS = S_RACK;
                              end
                           else
                              begin
                                 rNS = S_RSTART;
                              end
                        end
            S_RACK   :  begin
                           rNS = S_RCOMP;
                        end
            S_RCOMP  :  begin
                           if (PLB_MnRdDAck)
                              begin
                                 rNS = S_ICOUNT;
                              end
                           else
                              begin
                                 rNS = S_RCOMP;
                              end
                        end
            S_ICOUNT :  begin
                           if (rCounter == {(C_CSIZE){1'b1}})
                              begin
                                 rNS = S_ISTART;
                              end
                           else
                              begin
                                 rNS = S_ICOUNT;
                              end
                        end
            S_ISTART :  begin
                           if (PLB_MnAddrAck)
                              begin
                                 rNS = S_IACK;
                              end
                           else
                              begin
                                 rNS = S_ISTART;
                              end
                        end
            S_IACK   :  begin
                           rNS = S_ICOMP;
                        end
            S_ICOMP  :  begin
                           if (PLB_MnWrDAck)
                              begin
                                 rNS = S_WCOUNT;
                              end
                           else
                              begin
                                 rNS = S_ICOMP;
                              end
                        end
            default  :     rNS = S_IDLE;
         endcase
      end

   always @ (posedge clk)
      begin
         if (rst)
            begin
               rCS      <= S_IDLE;
               rCounter <= {(C_CSIZE){1'b0}};
               rRequest <= 1'b0;
               rRNW     <= 1'b0;
               rABus    <= 32'b0;
               rWrDBus  <= 32'b0;
               rStore   <= 32'b0;
            end
         else
            begin
               rCS <= rNS;

               if ((rCS == S_WCOUNT) || (rCS == S_RCOUNT) || (rCS == S_ICOUNT))
                  begin
                     rCounter <= rCounter + 1;
                  end
               else if ((rCS == S_WACK) || (rCS == S_RACK) || (rCS == S_IACK))
                  begin
                     rCounter <= {(C_CSIZE){1'b0}};
                  end

               if ((rCS == S_WSTART) || (rCS == S_RSTART) || (rCS == S_ISTART))
                  begin
                     rRequest <= 1'b1;
                  end
               else 
                  begin
                     rRequest <= 1'b0;
                  end

               case (rCS)
                  S_WSTART :  begin
                                 rABus <= C_WABUS;
                                 rWrDBus <= C_WDBUS;
                              end
                  S_RSTART :  begin
                                 rABus <= C_RABUS;
                                 rWrDBus <= 32'h0000_0000;
                              end
                  S_ISTART :  begin
                                 rABus <= C_IABUS;
                                 rWrDBus <= rStore + 1;
                              end
                  default  :  begin
                                 rABus <= 32'h0000_0000;
                                 rWrDBus <= 32'h0000_0000;
                              end
               endcase

               if (PLB_MnRdDAck)
                  begin
                     rStore <= PLB_MnRdDBus;
                  end
            end
      end

 endmodule

