module PLB_rw_FSM(
					done,
					rst,				//reset
					clk,				//FSM clock
					go,					//go signal
					ranw,				//read-not-write (controls transaction type)
					M_size,				//controls how many dwords written/read
					
					RNW,				//output Read-Not-Write signal to PLB
					Mn_wrBurst,			//output to PLB
					data_index,			//input/output data index. Indicates which data dword is being read/written
					mn_request_set,
					read_data,			//signal to get next read data ready (use the RdDack signal for WEN)
					write_data,			//signal to change the data on the PLB write bus.
					
					PLB_MWrDack,
					MnRdDack
					
				);

input 			rst, clk, go, ranw;
input	[3:0]	M_size;	

input			PLB_MWrDack;
input			MnRdDack;

output			done;
output 			RNW;
output 			Mn_wrBurst;
output	[3:0]	data_index;
output			mn_request_set;
output			read_data;				//toggle signal to read data from PLB bus
output			write_data;				//toggle signal to write data to PLB bus

//state registers
reg	[3:0]	state;
reg 			RNW;
reg 			Mn_wrBurst;
reg 	[3:0]	data_index;

//state machine transition
always@( posedge clk ) begin
	if( rst )
		state <= 0;
	else begin
		case( state )
		0: 	begin
			if( go && ~ranw )
				state <= 1;
			if( go && ranw )
				state <= 5;
			end
		1:	begin
			RNW <= 0;			//control signals
			Mn_wrBurst <= 1;	
			data_index <= 0;	//first data element selected
			state <= 2;
			end
		2:	begin
			if( data_index > M_size )
				state <= 0;
			else if( PLB_MWrDack )
				state <= 3;
			end
		3:	begin
			data_index <= data_index + 1;
			state <= 2;
			end
		4:	begin
			state <= 2;
			end
		5:	begin
			RNW <= 1;
			Mn_wrBurst <= 0;
			data_index <= 0;
			state <= 6;
			end
		6:	begin
			if( data_index > M_size )
				state <= 0;
			else if( MnRdDack )
				state <= 7;
			end
		7:	begin
			data_index <= data_index + 1;
			state <= 6;
			end
		8:	begin
			state <= 6;
			end
		default:
			state <= 0;
		endcase
	end
end

//output assignments
assign done				= (state == 0);
assign write_data 		= (state == 3);
assign read_data 		= (state == 7);
assign mn_request_set 	= ( (state==1) || (state==5) );
				
endmodule
					