`timescale 1 ns / 100 ps
module plb_read_test (
	//LED
	EQ,

  // PLB GLOBAL SIGNALS
  SYS_plbClk,        // I  100MHz
  SYS_plbReset,      // I 

  // REQUEST QUALIFIERS INPUTS
  PLB_MnAddrAck,     // I
  PLB_MnRearbitrate, // I
  PLB_Mnssize,       // I [0:1]
  PLB_MnBusy,        // I
  PLB_MnErr,         // I
  PLB_pendReq,       // I
  PLB_pendPri,       // I [0:1]
  PLB_reqPri,        // I [0:1]    
  Mn_request,        // O
  Mn_priority,       // O [0:1]
  Mn_busLock,        // O 
  Mn_RNW,            // O
  Mn_BE,             // O [0:7]
  Mn_size,           // O [0:3]
  Mn_type,           // O [0:2]

  Mn_msize,          // O [0:1]          
  Mn_compress,       // O        
  Mn_guarded,        // O
  Mn_lockErr,        // O
  Mn_ordered,        // O
  Mn_abort,          // O
  Mn_ABus,           // O [0:31]

  // PLB WRITE DATA BUS
  PLB_MnWrDAck,      // I       
  PLB_MnWrBTerm,     // I
  Mn_wrBurst,        // O
  Mn_wrDBus,         // O [0:63]

  // PLB READ DATA BUS              
  PLB_MnRdDAck,      // I
  PLB_MnRdBTerm,     // I
  PLB_MnRdWdAddr,    // I [0:3]
  PLB_MnRdDBus,      // I [0:63]       
  Mn_rdBurst        // O
  );
  //LED
  output EQ;
  
  // PLB GLOBAL SIGNALS
  input        SYS_plbClk;		// i
  input        SYS_plbReset;	// i

  // PLB REQUEST QUALIFIERS INPUTS
  input         PLB_MnAddrAck;	// i
  input         PLB_MnRearbitrate;
  input  [0:1]  PLB_Mnssize;
  input         PLB_MnBusy;
  input         PLB_MnErr;
  input         PLB_pendReq;
  input  [0:1]  PLB_pendPri;
  input  [0:1]  PLB_reqPri;     
  output        Mn_request;		// o
  output [0:1]  Mn_priority;	// o-
  output        Mn_busLock;			//
  output        Mn_RNW;			// o-
  output [0:7]  Mn_BE;			// o-
  output [0:3]  Mn_size;        // o-
  output [0:2]  Mn_type;		// o-
  output [0:1]  Mn_msize;       // o-      
  output        Mn_compress;        //      
  output        Mn_guarded;    		//
  output        Mn_ordered;     	//
  output        Mn_lockErr;      	//
  output        Mn_abort;      		//
  output [0:31] Mn_ABus;		// o

  // PLB READ DATA BUS
  input         PLB_MnRdDAck;   // i
  input         PLB_MnRdBTerm;      
  input [0:3]   PLB_MnRdWdAddr;	// i
  input [0:63]  PLB_MnRdDBus;   // i
  output        Mn_rdBurst;      	//
  // PLB WRITE DATA BUS
  input         PLB_MnWrDAck;       
  input         PLB_MnWrBTerm;      
  output        Mn_wrBurst;          
  output [0:63] Mn_wrDBus;        
  
  wire 			clk, rst;
  wire        	mn_request_set;
  reg   [0:23]  tft_base_addr_i;
  
  reg           get_line;			// i
  //reg   [0:10]  tft_base_addr;		// i
  reg   [0:23]  tft_base_addr;		// i
  reg           tft_on_reg;			// i
  wire			mn_request_set_reg;
  wire			rnw;
  
  assign Mn_busLock  = 1'b0;
  assign Mn_compress = 1'b0;
  assign Mn_guarded  = 1'b0;
  assign Mn_ordered  = 1'b0;
  assign Mn_lockErr  = 1'b0;
  assign Mn_abort    = 1'b0;  
  //assign Mn_wrBurst  = 1'b1;
  //assign Mn_wrDBus   = 64'b0;          
  assign Mn_rdBurst  = 1'b0;

  //assign Mn_MSize     = 2'b01;             // 64 Bit PLB Xfers 
  assign Mn_msize     = 2'b01;             // 64 Bit PLB Xfers 
  assign Mn_priority  = 2'b11;             // Set priority to 3
  assign Mn_size      = 4'b0001;           // Transfer 8-word line 
  assign Mn_type      = 3'b000;            // Memory type transfer
  //assign Mn_RNW       = 1'b0;              // Always read
  assign Mn_BE        = 8'b00000000;       // Ignored on Line xfers
  //assign Mn_ABus[0:10]  = tft_base_addr_i; // 11-bits
  assign Mn_ABus[0:23]  = tft_base_addr_i; // 11-bits
  //assign Mn_ABus[11:19] = line_cnt_i;
  //assign Mn_ABus[20:26] = trans_cnt_i;
  
  //make the temp bus bigger to allow for more flexibility
  reg	[0:7] temp_bus;
  assign Mn_ABus[24:31] = temp_bus;
  
  assign clk = SYS_plbClk;
  assign rst = SYS_plbReset;

  assign Mn_RNW       = rnw;
  //assign Mn_wrDBus   = 64'hdead_beef_cafe_babe; 
	reg [0:63] Mn_wrDBus;
  //assign mn_request_set = tft_on_reg & (  (get_line & (trans_cnt == 0))
  //                                      | (end_xfer & (trans_cnt != 0)));  
  assign mn_request_set = mn_request_set_reg;
  FDRSE FDRS_MN_REQUEST_DLY (.Q(Mn_request),.CE(1'b0),.C(clk),.D(1'b0),
                             .R(PLB_MnAddrAck | rst), .S(mn_request_set));

  always @(posedge clk)
    if (mn_request_set) begin
      tft_base_addr_i <= tft_base_addr;
      //line_cnt_i      <= line_cnt;
      //trans_cnt_i     <= trans_cnt;
    end   

//scrap code to make the FSM below do something interesting (for testing!)

//currently does the following:
//	at time 30M, writes two dwords to address zero. Writes 0xDEADBEEF_CAFEBABE
//	at time 60M, reads one dword from address b'0001_0000, replicating it to data1 and data2
//	at time 100M, writes data1 and data2 to address b'1000_0000

//currently in test mode

reg [26:0] counter;
reg do_once;
wire wgo = ( counter == 30_000_000 ) || (counter == 60_000_000) || (counter == 100_000_000);
//wire wgo = ( counter == 30 ) || (counter == 60) || (counter == 100);
//wire wgo = ( counter == 30 ) || (counter == 60) || (counter == 100);
wire		done;
wire [3:0]	data_index;
wire		read_now;
wire		write_now;
reg 		PLB_rw_FSM_go;
reg			PLB_rw_FSM_rnw;

reg [0:63] data1;
reg [0:63] data2;


always@( posedge SYS_plbClk ) begin
	if( SYS_plbReset ) begin
		tft_base_addr <= 0;
		counter <= 1;
		data1 <= 0;
		data2 <= 0;
		PLB_rw_FSM_go <= 0;
		PLB_rw_FSM_rnw <= 0;
		do_once <= 1;
		temp_bus <= 'h20;
		Mn_wrDBus <= 64'hdead_beef_cafe_babe; 
	end
	else
		begin
			if( PLB_MnRdDAck ) begin
				if( data_index == 0 )
					data1 <= PLB_MnRdDBus;
				if( data_index == 1 )
					data2 <= PLB_MnRdDBus;
			end
			
			if( do_once )
				counter <= counter + 1;
			if( counter == 0 )
				do_once <= 0;
			if( counter == 40_000_000 ) begin
			//if( counter == 40 ) begin
				temp_bus <= 'h40;	//read from this address
				PLB_rw_FSM_rnw <= 1;
			end
			if( counter == 80_000_000 ) begin	//write to PLB address 8
			//if( counter == 80 ) begin
				PLB_rw_FSM_rnw <= 0;
				temp_bus <= 'h80;	//write to this address (the contents of the address read above)
				Mn_wrDBus <= data1;
			end
		end
end	
	
assign EQ = ( data1 == data2 );

	
PLB_rw_FSM	FSM	(	.done				( done					),
					.rst				( SYS_plbReset			),				//reset
					.clk				( SYS_plbClk 			),				//FSM clock
					.go					( wgo ),				//go signal
					.ranw				( PLB_rw_FSM_rnw 		),				//read-not-write (controls transaction type)
					.M_size				( Mn_size 				),				//controls how many dwords written/read
					
					.RNW				( rnw 					),				//output Read-Not-Write signal to PLB
					.Mn_wrBurst			( Mn_wrBurst	 		),				//output to PLB
					.data_index			( data_index 			),			//input/output data index. Indicates which data dword is being read/written
					.mn_request_set		( mn_request_set_reg ),
					.read_data			( read_now ),
					.write_data			( write_now ),
					
					.PLB_MWrDack		( PLB_MnWrDAck ),
					.MnRdDack			( PLB_MnRdDAck )
				);




	
	initial
		begin
			
			/*
			tft_base_addr <= 27'h0000000;
			#400
			PLB_rw_FSM_rnw <= 0;
			PLB_rw_FSM_go  <= 1;
			#10
			PLB_rw_FSM_go <= 0;
			#400
			PLB_rw_FSM_rnw <= 1;
			PLB_rw_FSM_go  <= 1;
			#10
			PLB_rw_FSM_go  <= 0;
			*/
			/*
			mn_request_set_reg <= 1'b0;
			rnw <= 1'b0;
			
			#1600;
			mn_request_set_reg <= 1'b1;
			#10
			mn_request_set_reg <= 1'b0;		
			
			#300;
			rnw <= 1'b1;
			#50;
			mn_request_set_reg <= 1'b1;
			#10
			mn_request_set_reg <= 1'b0;	
			*/
		end
		

 endmodule