`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:35:20 03/13/2008 
// Design Name: 
// Module Name:    list_cluster_FSM 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module list_cluster_FSM02(	list1_addr,				//point1 address line
									list1_indata,			//input data for reading from point1 memory 
									list1_outdata,			//output data for writing to point1 memory 
									list1_en,				//duh.
									list1_wen,
									list1_clk,
									list1_ssr,
			
									list2_addr,				//point1 address line
									list2_indata,			//input data for reading from point1 memory
									list2_outdata,			//output data for writing to point1 memory 
									list2_en,				//duh.
									list2_wen,
									list2_clk,
									list2_ssr,
									
									BRAM_Addr,
									BRAM_Clk,				//BRAM control signals
									BRAM_Data,
									BRAM_Write,
									
									mclk,						//module clock -- powers BRAM and FSL
									go,						//signal to do the list / cluster / send to FSL operation
									reset						//global reset
								);
//PARAMETERS get declared first!
parameter DIST_THRESH 		= 30;
parameter CLUSTER_THRESH 	= 30;
parameter MAX_ELEMENTS		= 512;								
								
								
//global signals								
input mclk;									//module clock -- powers BRAM and FSL
input go;   								//signal to do the list / cluster / send to FSL operation
input reset;								//global reset

//BRAM access ports
output [8:0]	list1_addr;				//point1 address line
input  [31:0]	list1_indata;			//input data for reading from point1 memory
output [31:0]	list1_outdata;			//output data for writing to point1 memory 
output			list1_en;				//duh.
output			list1_wen;
output			list1_clk;
output			list1_ssr;


output [8:0]	list2_addr;				//point1 address line
input  [31:0]	list2_indata;			//input data for reading from point1 memory
output [31:0]	list2_outdata;			//output data for writing to point1 memory 
output			list2_en;				//duh.
output			list2_wen;
output			list2_clk;
output			list2_ssr;

//FSL IS BIG ENDIAN. From the spec:
//   MSB:LSB
//     0:C_FSL_DWIDTH-1

//Should also set C_FSL_DEPTH to 512, to accommodate the possibility that all the
//possible points are stuffed into the FSL before the uB reads any.

output	[8:0]		BRAM_Addr;
output				BRAM_Clk;			//FSL control signals - see FSL documentation
output	[0:31]	BRAM_Data;			//data to send to FSL link
output				BRAM_Write;		//Write enable signal



//Point data in BRAM is stored across a 32-bit wide register.

/*
First nine bits are the n coordinate (rows.) Next ten bits are the m
coordinate (columns.) These are stored little endian.
32-bit register
0      78 9     15 18    23       31
|------|| ------||------| |------|
[   n   ] [    m   ] 
LSB   MSB LSB	  MSB 

pointn = BRAM_data [8:0]
pointm = BRAM_data [18:9]
*/

//state registers
	//FSM state register
	reg [3:0] state;
	//current min element state
	reg [8:0] current;
	//offset element state
	reg [4:0] i;

//useful wires!
	wire current_iszero;
	wire [10:0] distance;
	wire [9:0]	midpm;	//ONE TOO MANY BITS!!
	wire [8:0]	midpn;
	//old declaration -- ENDIAN ISSUE
	//wire [31:0] midp_data = {midpn,midpm,13'b0};
	wire [31:0] midp_data = {13'b0, midpm, midpn};
	
//wires to allow flipping of addr/data buses for BRAMs
	//flip the address/data buses of the BRAMs to allow updating both
	wire flip;
	//disable midpoint module (stop it from updating) when writing to BRAM
	wire midp_EN;

	wire [31:0] BRAM1_dataout = flip ?  midp_data:32'b0;
	wire [8:0]	BRAM1_addrout = flip ? (current+i):current;

	wire [31:0] BRAM2_dataout = flip ? 32'b0:midp_data;
	wire [8:0]	BRAM2_addrout = flip ? current:(current+i);

//assign clock signals!
	assign list1_clk = mclk;
	assign list2_clk = mclk;

//BRAMs are always enabled
	assign list1_en = 1'b1;
	assign list2_en = 1'b1;
//BRAM addressing
	assign list1_addr = BRAM1_addrout;
	assign list2_addr = BRAM2_addrout;
//BRAM write data is always the same
	assign list1_outdata = BRAM1_dataout;
	assign list2_outdata = BRAM2_dataout;

//STATE MACHINE TRANSITION
always@( posedge mclk ) begin
if( reset ) begin
	state <= 0;
	current <= 0;
	i <= 0;
end
else begin
case( state )
	0:	begin	//start state
			//state assignment
			current <= 0;
			i <= 1;
			//transition logic
			if( go )
				state <= 12;
		end
		
	1:	begin	 	//do state
			//state assignment
			
			//transition logic								this case handles when i+current exceeds the address space
			if( current_iszero | (i > DIST_THRESH-1) | (i+current > MAX_ELEMENTS-2) )			//TEST THIS!! TEST IT! AHHHHH! (not sure about > statement)
				state <= 5;
			else						//if the current coordinate isn't zero, 
				state <= 2;			//and there's still points to be checked, keep going!
		end
		
	2:	begin	 	//distance state
			if( distance > CLUSTER_THRESH ) //points are too far apart to combine.
				state <= 3;
			else						//if the distance is smaller than the clustering 
				state <= 4;			//threshold, combine the two points and move on.
			
		end
		
	3:	begin	 	//not_found state
			//state assignment
			i <= i+1;
			//transition logic	
			state <= 12;
		end
		
	4:	begin	 	//write 
			state <= 11;
		end
		
	5:	begin		//increment curent
			//state assignment
			current <= current + 1;
			i <= 1;
			//transition logic
			if( current < MAX_ELEMENTS-2 )
				state <= 12;
			else
				state <= 6;
		end
		
	6:	begin		//reset_current for FSL writes
			//state assignment
			current <= 0;
			//transition logic
			state <= 13;
		end
		
	7:	begin		//FSL_write loop
			//transition logic
			if( current > MAX_ELEMENTS-2 )
				state <= 10;
			else if( current_iszero )
				state <= 8;
			else if( ~current_iszero )
				state <= 9;
		end
		
	8:	begin		//
			//state assignment
			current <= current + 1;
			//transition logic
			state <= 13;
		end
		
	9: begin
			//transition logic
			state <= 8;
		end	
		
	10:begin
			//transition logic
			state <= 0;
		end
	
	11:begin	//write mid to current+i
			//transition logic
			state <= 5;
		end
		
	12:begin	//wait states! See notes
			state <= 1;
		end
	
	13:begin
			state <= 7;
		end
		
	default:
		state <= 0;	
endcase
end	//else statement
end	//always@ statement

//State-based combinatorial outputs

//midpoint enable signal
assign midp_EN			= ~( (state==4)|(state==11) );
//BRAM data/address bus choice
assign flip 			= ( state == 11 );
//BRAM write enables
assign list1_wen 		= ( state == 4 )|( state == 11 );
assign list2_wen 		= ( state == 4 )|( state == 11 );
//BRAM reset
assign list1_ssr 		= ( state == 10 );
assign list2_ssr 		= ( state == 10 );
//BRAM
assign BRAM_Write 	= ( state == 9 );//Assign BRAM signals
assign BRAM_addr = current;
assign BRAM_Clk = mclk;
assign BRAM_Data = list1_indata;
assign BRAM_Control = 1'b0;


//Combinational (operational) sub-blocks

	//this block indicates whether the current coordinate is zero.
	point_iszero tellzero1 (
		 .point		( list1_indata ), 
		 .iszero	( current_iszero )
		 );

	//this block calculates the distance between the two points presented
	//at list1_indata and list2_indata
	distance dist (
		.point1m		( list1_indata[18:9] ), 
		.point1n		( list1_indata[8:0]  ), 
		.point2m		( list2_indata[18:9] ), 
		.point2n		( list2_indata[8:0]  ), 
		.distance		( distance 			 )	
		);

	//this block calculates the midpoint between two points.
	midpoint mid (
    .point1m		( list1_indata[18:9] ), 
    .point1n		( list1_indata[8:0]  ), 
    .point2m		( list2_indata[18:9] ), 
    .point2n		( list2_indata[8:0]  ), 
    .midpm			( midpm				), 
    .midpn			( midpn				),
	.EN				( midp_EN			),
	.clk			( mclk				)
    );

endmodule
