`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    23:29:15 03/12/2008 
// Design Name: 
// Module Name:    midpoint 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
/*
This module calculates the midpoint of two points. See the Matlab code in blob_cluster.m
for more details. 

It is assumed that the synthesis tool will detect that the same logic is reused here and
in the distance() module, so it will be duplicated shamelessly!

This is the relevant Matlab code (taken from blob_cluster.m):

    %Set the new elements
    blist_cluster( bi+i, 1 ) = cast( blist_cluster( bi, 1 ) + xdist/2, 'int16');
    blist_cluster( bi+i, 2 ) = cast( blist_cluster( bi, 2 ) + ydist/2, 'int16');
	 
where bi is point1 and bi+i is point2.

ALL INPUTS/OUTPUTS ARE ASSUMED TO BE UNSIGNED.
*/
module midpoint(	point1m, point1n,		//first point input
						point2m, point2n, 	//second point input
						midpm, midpn,			//midpoint output
						EN,clk					//Enable signal,clk
						);

input  [9:0] 	point1m, point2m;
input  [8:0] 	point1n, point2n;
input			EN, clk;
output [9:0]	midpm;	//outputs midpm/n are one bit larger to accommodate possible carry-out
output [8:0]	midpn;	//in assign statement below.

wire signed [10:0] 	distancem;
wire signed [9:0] 	distancen;

wire signed [10:0] 	midm;
wire signed [9:0] 	midn;

wire signed [9:0] divm;
wire signed [8:0] divn;

wire coutm;
wire coutn;

wire [9:0] midpmw;
wire [8:0] midpnw;

reg [9:0]	midpm = 0;	//outputs midpm/n are one bit larger to accommodate possible carry-out
reg [8:0]	midpn = 0;	//in assign statement below.

//Calculate the distance between points.
distance_comp dcomp (
    .point1m	( point1m   ), 
    .point1n	( point1n   ), 
    .point2m	( point2m   ), 
    .point2n	( point2n   ), 
    .distancem	( distancem ), 
    .distancen	( distancen )
    );

assign divm = distancem >> 1;
assign divn = distancen >> 1;

assign midm = {distancem[10], divm};	//10 bits
assign midn = {distancen[9],  divn};

assign {coutm, midpmw} = point1m + midm;	//10+10 bits
assign {coutn, midpnw} = point1n + midn;	//9+9 bits

always@( posedge clk ) begin
	if( EN ) begin
		midpm <= midpmw; //drop the carry-out, since it is guaranteed the values can't overflow?
		midpn <= midpnw;
	end
end

endmodule


/* OLD CODE
assign divm = distancem >> 1;
assign divn = distancen >> 1;

assign midm = {distancem[10], divm};	//10 bits
assign midn = {distancen[9],  divn};

assign {coutm, midpmw} = point1m + midm;	//10+10 bits
assign {coutn, midpnw} = point1n + midn;	//9+9 bits

assign midpm = midpmw; //drop the carry-out, since it is guaranteed the values can't overflow?
assign midpn = midpnw;
*/