`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    00:23:04 03/19/2008 
// Design Name: 
// Module Name:    top 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
/* This file contains the same top module as top.v, but without the PLB interface signals.
It is used to do an end-to-end test from a loaded BRAM buffer of 12x10 pixels.
*/
module top3 (	
			  SYS_plbClk,        // I  100MHz
			  SYS_plbReset,      // I 
			  //master_go,
			  //Capture core interface			
			  HCAP_clk,			//I clock for HCAP port of BRAM
			  //Results output interface
			  BRAM_Addr,
			  BRAM_Clk,
			  BRAM_Data,
			  BRAM_Write,
			  //Capture core data interface
			  oClk_vga,
			  oAddr_vga,
			  iData_vga,
			  iEol_vga,
			  iEof_vga
			);

//ports for data input from capture core
output            oClk_vga;
output   [9: 0]   oAddr_vga;
input    [7: 0]   iData_vga;
input             iEol_vga;
input             iEof_vga;
//BRAM output interface
output	[8:0]		BRAM_Addr;
output				BRAM_Clk;
output	[31:0]	BRAM_Data;
output				BRAM_Write;

//GLOBAL SIGNALS
input		  		SYS_plbClk;        // I  100MHz
input		  		SYS_plbReset;      // I 
//input				master_go;

//capture core interface
wire	[95:0]	HCAP_data;
wire				HCAP_wen;
input 			HCAP_clk;
wire				line_pulse;
wire				frame_pulse;

//WIRES - all the wires that have to go to the control FSM
wire				master_go;
wire				shifter_go;
wire 				cluster_master_go;
wire				shifter_done;
wire 	[8:0]	  	pointn;
wire 	[9:0]	   pointm;			
wire	[8:0]		SDRAM_addr;

//WIRES - internal
wire  [959:0] 	M;
wire	[527:0] 	dM;
wire	[23:0]	islocal_min;
wire	[8:0]		source_addr;
wire	[31:0]	source_data;
wire				source_clk;
wire				source_en;
wire				source_wen;
wire	[8:0]		list1_addr;				//point1 address line
wire	[31:0]	list1_write;			//input data for writing to point1 memory
wire	[31:0]	list1_read;			//output data for reading from point1 memory
wire				list1_en;				//duh.
wire				list1_wen;
wire				list1_clk;
wire				list1_ssr;
wire	[8:0]		list2_addr;				//point1 address line
wire	[31:0]	list2_write;			//input data for writing to point1 memory
wire	[31:0]	list2_read;			//output data for reading from point1 memory
wire				list2_en;				//duh.
wire				list2_wen;
wire				list2_clk;
wire				list2_ssr;
wire				FSL_M_Clk;
wire 	[31:0]	FSL_M_Data;
wire 				FSL_M_Control;
wire 				FSL_M_Write;
wire 				FSL_M_Full;



//wires to make M matrix
wire	[95:0]		line0 ;
wire	[95:0]		line1 ;
wire	[95:0]		line2 ;
wire	[95:0]		line3 ;
wire	[95:0]		line4 ;
wire	[95:0]		line5 ;
wire	[95:0]		line6 ;
wire	[95:0]		line7 ;
wire	[95:0]		line8 ;
wire	[95:0]		line9 ;


assign M = { line0,line1,line2,line3,line4,line5,line6,line7,
			 line8,line9 };

/* To test this soup, we need a top-level FSM. This module will do for now. 
The test will consist of an image that is approximately 110 pixels on a side.*/
master_FSM02 master (
	.shifter_go				( shifter_go 			),
	.cluster_master_go	( cluster_master_go 	),
	.shifter_done			( shifter_done 		),
	.pointn					( pointn		 			),
	.pointm					( pointm 				),
	.SDRAM_addr				( SDRAM_addr 			),	
	.clk						( SYS_plbClk 			),
	.master_go				( master_go 			),
	.rst						( SYS_plbReset	|| frame_pulse	) //reset base address when new frame is started
	);


//Input block
data_transfer data_in (
    .iClk_pd				( SYS_plbClk			), 
    .iRst_pd				( SYS_plbReset			), 
    .oData_pd				( HCAP_data				), 
    .oGo_pd					( HCAP_wen				), 
    .oEol_pd				( line_pulse			), 
    .oEof_pd				( frame_pulse			), 
	 
    .oClk_vga				( oClk_vga	), 		//output ports
    .oAddr_vga				( oAddr_vga	), 
    .iData_vga				( iData_vga	), 
    .iEol_vga				( iEol_vga	), 
    .iEof_vga				( iEof_vga	)
    );


/*This block is the main block buffer for the module. The source image
data gets copied in blocks of 12(w) by 10(h) pixels, for a total of 24
processed pixels at the end. This is because the identification of a pixel
as a local minimum requires a grid of 7x7 pixels. Thus, the pixels at the
edge of the buffer are not actually worked on, but needed to compute is_min
for those in the centre. See the work notes for further information.
*/

bram_buffer2 buffer (
	.HCAP_data			( HCAP_data 				),			//I [95:0], input data from PLB SDRAM reader module
	.HCAP_wen			( HCAP_wen 					),			//I straight-through to BRAM
	//.HCAP_line_sel		( HCAP_line_sel ),				//I [3:0] picks one of 11 line numbers
	//.HCAP_pix_addr		( HCAP_pix_addr ),				//I [5:0] picks block of 12 pixels in a given line							
	.HCAP_clk			( HCAP_clk 					),			//I clock for HCAP port of BRAM
	.line_pulse			( line_pulse 				),			//I at the end of every line
	.frame_pulse		( frame_pulse				),
	.oline0				( line0 						), 
	.oline1				( line1  					), 
	.oline2				( line2 						), 
	.oline3				( line3 						), 
	.oline4				( line4  					), 
	.oline5				( line5  					), 
	.oline6				( line6  					),
	.oline7				( line7  					),
	.oline8				( line8  					),
	.oline9				( line9  					), 	
	.M_proc_ready		( master_go 				),		//O one cycle pulse to say that a block of 10 lines is ready for processing at online0-9.
	.M_matrix_addr		( SDRAM_addr 				),		//I [8:0] Address
	.M_matrix_clk		( SYS_plbClk 				),		//I
	.rst					( SYS_plbReset 			)		//global reset signal
	);
	
/*This block computes the scalar del-squared terms for a block of pixels that
is a subset of the original block. Specifically, a block of 8(w) x 6(h) pixels
is computed. The results are actually 11 bits each, since the del-squared
algorithm is based on addition/subtraction (and therefore generates carry-outs.)
*/
del_squared3 del3 (
    .M						( M 						), 
    .dM						( dM 						)
    );
	
/*This block gives a boolean value for a block of 24 pixels (6(w) x 4(h)) which
indicates whether a given pixel is a local minimum or not. The calculations to
determine this are basically 8 comparisons to determine whether a given pixel
is smaller than all of its' neighbours. An array of 24 bits is outputted to the
islocalmin port.
*/

//WARNING -- dM IS ALMOST CERTAINLY CONNECTED INCORRECTLY.
islocal_min2 is_lmin (
    .dM						( dM 						), 
    .islocalmin			( islocal_min			) 
    //.go					( 'b0 					),	//this doesn't actually do anything 
    //.mclk					( 'b0		 				), //no functionality
    //.rst					( SYS_plbReset 		)	//no functionality
    );
	
/*This block takes the islocalmin matrix and the block address in pixels as input,
converting this simple output into ordered sets of points and inputting them
serially into the BRAMs used for the clustering algorithm.
*/
BRAM_shift shifter (
    .islocal_min			( islocal_min 			), 
    .pointn					( pointn 				),		//use first possible block 
    .pointm					( pointm					), 		//block numbering starts at one?
    .source_addr			( source_addr			), 
    .source_data			( source_data			), 
    .source_clk			( source_clk			), 	
    .source_en				( source_en				), 
    .source_wen			( source_wen			), 
    .clk						( SYS_plbClk 			), 
    .go						( shifter_go			), 
    .rst						( SYS_plbReset 		),
	 .done					( shifter_done			)
    );

/*This BRAM block holds the set of ordered points that describes all the point
detections made thus far. It is 512 words deep, and can hold up to as many 
detected points. The BRAMs are duplicate copies of one another, to allow for
simultaneous access to the data for processing (hence the list1 and list2
output ports.
*/
list_mem cluster_mem (
    .source_addr			( source_addr			),	//connections from BRAM_shift shifter
    .source_data			( source_data			), 
    .source_clk			( source_clk			), 
    .source_en				( source_en				), 
    .source_wen			( source_wen			),
    .list1_addr			( list1_addr			), 
    .list1_indata			( list1_write			), 
    .list1_outdata		( list1_read			), 
    .list1_en				( list1_en				), 
    .list1_wen				( list1_wen				), 
    .list1_clk				( list1_clk				), 
    .list1_ssr				( list1_ssr				), 
    .list2_addr			( list2_addr			), 
    .list2_indata			( list2_write			), 
    .list2_outdata		( list2_read			), 
    .list2_en				( list2_en				), 
    .list2_wen				( list2_wen				), 
    .list2_clk				( list2_clk				), 
    .list2_ssr				( list2_ssr				)
    );

/* Finally, this module clusters the detected points, and pumps the resulting data
out to an FSL. The FSL interface is assumed to have a depth of at least 512 words,
so that FIFO overflow conditions never occur. This, however, may change if there 
are issues with capacity on the FPGA. 
*/
list_cluster_FSM02 cluster_master (
    .list1_addr			( list1_addr 			), 
    .list1_indata			( list1_read			), 	//all list read/write are named from the
    .list1_outdata		( list1_write			), 	//FSM's point of view.
    .list1_en				( list1_en				), 
    .list1_wen				( list1_wen				), 
    .list1_clk				( list1_clk				), 
    .list1_ssr				( list1_ssr				), 
    .list2_addr			( list2_addr			), 
    .list2_indata			( list2_read			), 
    .list2_outdata		( list2_write			), 
    .list2_en				( list2_en				), 
    .list2_wen				( list2_wen				), 
    .list2_clk				( list2_clk				), 
    .list2_ssr				( list2_ssr				), 
	 .BRAM_Addr				( BRAM_Addr			 	),
    .BRAM_Clk				( BRAM_Clk				), 
    .BRAM_Data				( BRAM_Data				), 			//create FSL interface in this file
    .BRAM_Write			( BRAM_Write			), 
    .mclk					( SYS_plbClk 			),     
	 .go						( cluster_master_go 	), 
    .reset					( SYS_plbReset 		)
    );

endmodule
