module vidcap(
	BRAM_Rst_B,
	BRAM_Clk_B,
	BRAM_EN_B,
	BRAM_WEN_B,
	BRAM_Addr_B,
	BRAM_Din_B,
	BRAM_Dout_B,

	YCrCb_in,
	LLC_CLOCK,
	RESET_VDEC1_Z,
	VDEC1_OE_Z,
	VDEC1_PWRDN_Z,

	BRAM_Rst_PD,
	BRAM_Clk_PD,
	BRAM_EN_PD,
	BRAM_WEN_PD,
	BRAM_Addr_PD,
	BRAM_Din_PD,
	BRAM_Dout_PD
);

input			BRAM_Rst_B;
input			BRAM_Clk_B;
input			BRAM_EN_B;
input	[0:7]	BRAM_WEN_B;
input	[0:31]	BRAM_Addr_B;
output	[0:63]	BRAM_Din_B;
input	[0:63]	BRAM_Dout_B;

input	[9:2]	YCrCb_in;
input			LLC_CLOCK;
output			RESET_VDEC1_Z;
output			VDEC1_OE_Z;
output			VDEC1_PWRDN_Z;

input			BRAM_Rst_PD;
input			BRAM_Clk_PD;
input			BRAM_EN_PD;
input	[0:7]	BRAM_WEN_PD;
input	[0:31]	BRAM_Addr_PD;
output	[0:63]	BRAM_Din_PD;
input	[0:63]	BRAM_Dout_PD;

wire			wVidMemClk;
wire	[0:63]	wVidMemData;
wire	[0:11]	wVidMemAddr;
wire			wVidMemWE;

wire	PDClock = wVidMemClk;
wire	[9:0]	wPDAddr;
wire	[7:0]	wPDData;
wire			wEOL, wEOF;

//Reads data from video stream and writes it to VidMem
vid2vidmem	VideoDecoder(
	.iYCrCb_in(YCrCb_in),
	.iLLC_CLOCK(LLC_CLOCK),
	.iReset(BRAM_Rst_B),
	.oRESET_VDEC1_Z(oRESET_VDEC1_Z),
	.oVDEC1_OE_Z(oVDEC1_OE_Z),
	.oVDEC1_PWRDN_Z(oVDEC1_PWRDN_Z),

	.oVidMemClk(wVidMemClk),
	.oVidMemData(wVidMemData),
	.oVidMemAddr(wVidMemAddr),
	.oVidMemWE(wVidMemWE),

	.iPDClk(PDClock),
	.iPDAddr(wPDAddr),
	.oPDData(wPDData),
	.oHSync(wEOL),
	.oVSync(wEOF)
);

//Memory to store a few lines for display
blk_mem_gen_v2_3 VidMem (
	.clka(wVidMemClk),
	.dina(wVidMemData),
	.addra(wVidMemAddr),
	.ena(1'b1),
	.wea(wVidMemWE),

	.clkb(BRAM_Clk_B),
	.addrb(BRAM_Addr_B[17:28]),
	.enb(1'b1),
	.ssrb(BRAM_Rst_B),
	.doutb(BRAM_Din_B)
);

wire	[9:0]	PDBRAM_Addr;
wire	[7:0]	PDBRAM_Data;
wire			PDBRAM_WE;

top3 PDCore (	
	.SYS_plbClk(PDClock),
	.SYS_plbReset(BRAM_Rst_B),
	//master_go,
	//Capture core interface			
	.HCAP_clk(PDClock),			//I clock for HCAP port of BRAM
	//Results output interface
	.BRAM_Addr(PDBRAM_Addr),
	.BRAM_Clk(PDClock),
	.BRAM_Data(PDBRAM_Data),
	.BRAM_Write(PDBRAM_WE),
	//Capture core data interface
	.oClk_vga(PDClock),
	.oAddr_vga(wPDAddr),
	.iData_vga(wPDData),
	.iEol_vga(wEOL),
	.iEof_vga(wEOF)
);

blk_mem_gen_16k_8 PDResults (
	.clka(PDClock),
	.dina(PDBRAM_Data),
	.addra(PDBRAM_Addr),
	.ena(1'b1),
	.wea(PDBRAM_WE),

	.clkb(BRAM_Clk_PD),
	.addrb(BRAM_Addr_PD),
	.enb(BRAM_EN_PD),
	.ssrb(BRAM_Rst_PD),
	.doutb(BRAM_Din_PD)
);
assign BRAM_Din_PD[0:31] = 32'd0;

endmodule
