#include "2D_types.h"
#include "collidable.h"
#include "ball.h"
#include "draw.h"

//#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include "main.h"

#define TIME 1
#define DELAY 250000000

#define HSIZE 640
#define VSIZE 480
#define BOFFSET 80
#define LOFFSET 120

//#define DRAG 0.98
//#define BOUNCE 1.15
#define DRAG 1
#define BOUNCE 1

//Redraw offscreen buffers (can be used to refresh buffers after updating vertical offset)
void RedrawBuffers(VGA_Buffer *Buffers, int NumBuffers) {
	int	x, y;

	//Initialize offscreen buffers
	for (x = 0; x < VGA_RES_X; x++) {
		for (y = 0; y < VGA_RES_Y; y++) {
			//Checkerboard
			VGA_PSet(Buffers[0], x, y, (((x / 32) ^ (y / 32)) & 1) ? VGA_COLOUR_WHITE : VGA_COLOUR_BLACK);
			//Vertical stripes
			VGA_PSet(Buffers[1], x, y, VGA_RGB2C((x % 32) * 8, 0, 0));
			//Horizontal stripes
			VGA_PSet(Buffers[2], x, y, VGA_RGB2C(0, (y % 32) * 8, 0));
			//Fancy pattern 1
			VGA_PSet(Buffers[3], x, y, x * y);
			//Fancy pattern 2
			VGA_PSet(Buffers[4], x, y, VGA_COLOUR_WHITE - (x * x + y * y));
		};
	};
};

void InitOffset(void) {
	VGA_Buffer	Buffers[5];
	int		NumBuffers = 5;
	Xuint32	Buttons;					//Push button states
	int	CurrBuff = 0;					//Current buffer being displayed
	int	x, y;
	int	CurrY = 0;
	int	i;

	print("Initializing screen offset, press up and down at the same time to exit\r\n");
	for (i = 0; i < 5; i++) {
		Buffers[i] = (VGA_Buffer) (0x00200000 * i);
	};

	RedrawBuffers(Buffers, NumBuffers);

	do {
		//Read push buttons
		Buttons = XGpio_mGetDataReg(XPAR_PUSHBUTTONS_5BIT_BASEADDR, 1) & 0x1F;
		//Detect button press
		if (Buttons != 0x1F) {
			print("B");
			//Shift screens accordingly
			if (!(Buttons & _BV(PB_R))) {
				print("r");
				RedrawBuffers(Buffers, NumBuffers);
			} else if (!(Buttons & _BV(PB_L))) {
				print("l");
				VGA_VOffset = CurrY;
				RedrawBuffers(Buffers, NumBuffers);
			} else if (!(Buttons & _BV(PB_D))) {
				print("d");
				CurrY++;
				if (CurrY >= VGA_RES_Y) {
					CurrY = 0;
				};
				usleep(10000);
			} else if (!(Buttons & _BV(PB_U))) {
				print("u");
				CurrY--;
				if (CurrY < 0) {
					CurrY = VGA_RES_Y - 1;
				};
				usleep(10000);
			} else if (!(Buttons & _BV(PB_C))) {
				print("!");
				CurrBuff = (CurrBuff + 1) % NumBuffers;
				//Debounce and wait for push button to be released
				usleep(10000);
				while (!(XGpio_mGetDataReg(XPAR_PUSHBUTTONS_5BIT_BASEADDR, 1) & _BV(PB_C))) {};
				//Redraw screen
				VGA_SetActiveBuffer(Buffers[CurrBuff]);
			};
			VGA_PSet(Buffers[CurrBuff], 0, CurrY, VGA_COLOUR_GREY);
			print("\r\n");
		};
	} while ((Buttons & _BV(PB_U)) || (Buttons & _BV(PB_D)));

	print("Done!\r\n");
};

//Initialize application environment
void Init(VGA_Buffer *Buffers, int NumBuffers) {
	int	i;

	print("\r\nProgram running.\n\r");

	//Set 5 LSB for input (5 push buttons)
	XGpio_mSetDataDirection(XPAR_PUSHBUTTONS_5BIT_BASEADDR, 1, 0x1F);

	for (i = 0; i < NumBuffers; i++) {
		Buffers[i] = (VGA_Buffer) (0x00200000 * i);
	};

//	RedrawBuffers(Buffers, NumBuffers);

	VGA_SetActiveBuffer(Buffers[0]);	//Display first buffer
	VGA_On();							//Turn screen on

	//Initialize screen offset
	print("Calling init\n\r");
	InitOffset();
};

main(void) {
   int i, j;
   int frame_count = 0;

   struct state sBall;
   struct wall wWall[4];

   struct point2 pNewPos, pColl, pPosNear, pNewPosNear;
   struct point2 pOldFramePos[2]; 
   struct vector2 vNewVel, vOffset, vIncid, uIncid, uReflect;

   float length_coll;

   int			NumBuffers = 2;
   VGA_Buffer	Frames[NumBuffers];		//Offscreen buffers

   struct timespec delay, remain;
   delay.tv_sec = 0;
   delay.tv_nsec = DELAY;

   print("Calling init\r\n");
   Init(Frames, NumBuffers);

   //Clear display
   for (i = 0; i < NumBuffers; i++)
   {
		VGA_Clear(Frames[i]);
   }

   wWall[0].pOrigin.x = 0;
   wWall[0].pOrigin.y = 0;
   wWall[0].uDir.x = 1;
   wWall[0].uDir.y = 0;
   wWall[0].fLength = 400;
   wWall[0].uNormal.x = 0;
   wWall[0].uNormal.y = 1;

   wWall[1].pOrigin.x = 400;
   wWall[1].pOrigin.y = 0;
   wWall[1].uDir.x = 0;
   wWall[1].uDir.y = 1;
   wWall[1].fLength = 300;
   wWall[1].uNormal.x = -1;
   wWall[1].uNormal.y = 0;

   wWall[2].pOrigin.x = 400;
   wWall[2].pOrigin.y = 300;
   wWall[2].uDir.x = -1;
   wWall[2].uDir.y = 0;
   wWall[2].fLength = 400;
   wWall[2].uNormal.x = 0;
   wWall[2].uNormal.y = -1;

   wWall[3].pOrigin.x = 0;
   wWall[3].pOrigin.y = 300;
   wWall[3].uDir.x = 0;
   wWall[3].uDir.y = -1;
   wWall[3].fLength = 300;
   wWall[3].uNormal.x = 1;
   wWall[3].uNormal.y = 0;

   sBall.pPos.x = 10;
   sBall.pPos.y = 28; 
   //sBall.vVel.x = -5; 
   //sBall.vVel.y = 30; 
   sBall.vVel.x = 1; 
   sBall.vVel.y = 1; 
   sBall.iRadius = 5;

   pOldFramePos[0].x = HSIZE/2;
   pOldFramePos[0].y = VSIZE/2;
   pOldFramePos[1].x = HSIZE/2;
   pOldFramePos[1].y = VSIZE/2;

   draw_init(Frames, NumBuffers);
   
   //for(j = 0; j < 50; j++)
   for(;;)
   {
      //clear_screen();
      frame_count = !frame_count;
      erase_ball(Frames[frame_count], (int)(pOldFramePos[frame_count].x + LOFFSET), (int)(pOldFramePos[frame_count].y + BOFFSET));
      draw_ball(Frames[frame_count], (int)(sBall.pPos.x + LOFFSET), (int)(sBall.pPos.y + BOFFSET));
	  VGA_SetActiveBuffer(Frames[frame_count]);
      //print_screen(frame[frame_count]);

      nanosleep(&delay, &remain);
      pNewPos = p_add_v(sBall.pPos, scale_v(sBall.vVel, TIME));
      vNewVel = scale_v(sBall.vVel, DRAG);

      for(i = 0; i < 4; i++)
      {

         vOffset = findOffset(sBall, wWall[i]);
         pPosNear = p_add_v(sBall.pPos, vOffset);
         pNewPosNear = p_add_v(pNewPos, vOffset);

         if (checkCrossing(wWall[i], pPosNear, pNewPosNear))
         {
            pColl = findCollisionPoint(wWall[i], pPosNear, pNewPosNear);

            if(length_v(p_minus_p(pColl, wWall[i].pOrigin)) < wWall[i].fLength)
            {
               vIncid = p_minus_p(pNewPosNear, pPosNear);
               length_coll = length_v(vIncid) - length_v(p_minus_p(pColl, pPosNear));
               uIncid = unit_v(vIncid);
               uReflect = reflectVector(wWall[i], uIncid);
               pNewPos = p_add_v(p_add_v(pColl, scale_v(uReflect, length_coll)), scale_v(vOffset, -1));
               vNewVel = scale_v(uReflect, BOUNCE*length_v(sBall.vVel));
            }
         }
      }
      
      pOldFramePos[frame_count] = sBall.pPos;
      sBall.pPos = pNewPos;
      sBall.vVel = vNewVel;
   }

   return 0;
};
