//******************************************************************************
//vgademo		-	Program to demonstrate the VGA driver library
//
//By			-	Yi Yao				994085583
//Date			-	2008-03-04
//******************************************************************************

#include "vgademo.h"


int main(void) {
	Xuint32	Buttons;					//Push button states
	int	CurrBuff = 0;					//Current buffer being displayed
	int	NumBuffers = 5;					//Number of offscreen buffers
	VGA_Buffer	Buffers[NumBuffers];	//Offscreen buffers
	int	x, y;
	int	CurrY = 0;

	Init(Buffers, NumBuffers);

	for (;;) {
		//Read push buttons
		Buttons = XGpio_mGetDataReg(XPAR_PUSHBUTTONS_5BIT_BASEADDR, 1) & 0x1F;
		//Detect button press
		if (Buttons != 0x1F) {
			print("B");
			//Shift screens accordingly
			if (!(Buttons & _BV(PB_R))) {
				print("r");
				RedrawBuffers(Buffers, NumBuffers);
			} else if (!(Buttons & _BV(PB_L))) {
				print("l");
				VGA_VOffset = CurrY;
				RedrawBuffers(Buffers, NumBuffers);
			} else if (!(Buttons & _BV(PB_D))) {
				print("d");
				CurrY++;
				if (CurrY >= VGA_RES_Y) {
					CurrY = 0;
				};
				usleep(10000);
			} else if (!(Buttons & _BV(PB_U))) {
				print("u");
				CurrY--;
				if (CurrY < 0) {
					CurrY = VGA_RES_Y - 1;
				};
				usleep(10000);
			} else if (!(Buttons & _BV(PB_C))) {
				print("!");
				CurrBuff = (CurrBuff + 1) % NumBuffers;
				//Debounce and wait for push button to be released
				usleep(10000);
				while (!(XGpio_mGetDataReg(XPAR_PUSHBUTTONS_5BIT_BASEADDR, 1) & _BV(PB_C))) {};
				//Redraw screen
				VGA_SetActiveBuffer(Buffers[CurrBuff]);
			};
			VGA_PSet(Buffers[CurrBuff], 0, CurrY, VGA_COLOUR_GREY);
			print("\r\n");
		};
	};

	return 0;
};


//Initialize application environment
void Init(VGA_Buffer *Buffers, int NumBuffers) {
	int	i;

	print("\r\nProgram running.\n\r");

	//Set 5 LSB for input (5 push buttons)
	XGpio_mSetDataDirection(XPAR_PUSHBUTTONS_5BIT_BASEADDR, 1, 0x1F);

	for (i = 0; i < NumBuffers; i++) {
		Buffers[i] = (VGA_Buffer) (0x00200000 * i);
	};

	RedrawBuffers(Buffers, NumBuffers);

	VGA_SetActiveBuffer(Buffers[0]);	//Display first buffer
	VGA_On();							//Turn screen on
};


//Redraw offscreen buffers (can be used to refresh buffers after updating vertical offset)
void RedrawBuffers(VGA_Buffer *Buffers, int NumBuffers) {
	int	x, y;

	//Initialize offscreen buffers
	for (x = 0; x < 640; x++) {
		for (y = 0; y < 480; y++) {
			//Checkerboard
			VGA_PSet(Buffers[0], x, y, (((x / 32) ^ (y / 32)) & 1) ? VGA_COLOUR_WHITE : VGA_COLOUR_BLACK);
			//Vertical stripes
			VGA_PSet(Buffers[1], x, y, VGA_RGB2C((x % 32) * 8, 0, 0));
			//Horizontal stripes
			VGA_PSet(Buffers[2], x, y, VGA_RGB2C(0, (y % 32) * 8, 0));
			//Fancy pattern 1
			VGA_PSet(Buffers[3], x, y, x * y);
			//Fancy pattern 2
			VGA_PSet(Buffers[4], x, y, VGA_COLOUR_WHITE - (x * x + y * y));
		};
	};
};
