//******************************************************************************
//vga			-	VGA driver library
//
//By			-	Yi Yao				994085583
//Last revision	-	2008-03-04
//
//Notes:
//	In the plb_tft_cntlr_ref_v1_00_d pcore, only the 11 MSB of the frame buffer
//		base address are kept. This means that each frame has an address space
//		of 21 LSB = 2MB. The beginning of each frame must align on a 2MB
//		boundary. This is an implementation detail which should be abstracted
//		and kept hidden by this library.
//	Lines start every 1024 bytes. Even though the horizontal resolution is only
//		640, the extra 384 bytes are wasted for the sake of lines aligning to
//		kilobyte boundaries.
//	To use this library, you may want to:
//		1 - Instantiate a plb_tft_cntlr_ref_v1_00_d on the PLB in your system
//		2 - Add vga.h and vga.c to your sources
//		3 - Declare an offscreen buffer aligned to a 2MB boundary such as:
//				VGA_Buffer	MyBuffer = (VGA_Buffer) 0x00200000;
//		4 - Clear the off screen buffer and fill it up to your heart's content:
//				VGA_Clear(MyBuffer);
//				VGA_PSet(MyBuffer, 10, 10, VGA_COLOUR_RED);
//				etc
//		5 - Activate the offscreen buffer and turn on the display:
//				VGA_SetActiveBuffer(MyBuffer);
//				VGA_On();
//		6 - Write offscreen displays and set them active when you need them,
//				there is no need to turn on the display for all subsequent
//				frames
//******************************************************************************

#include "vga.h"

int	VGA_VOffset		= 0;

//Sets the active buffer for display
void VGA_SetActiveBuffer(VGA_Buffer buffer) {
	XIo_Out32(0xD0000040, buffer);
};


//Enables VGA output
void VGA_On(void) {
	XIo_Out32(0xD0000044, 0x1);			//Main screen turn on
};


//Disables VGA output
void VGA_Off(void) {
	XIo_Out32(0xD0000044, 0x0);			//All your base are belong to us!
};


//Clears a display buffer to black
void VGA_Clear(VGA_Buffer buffer) {
	VGA_ClearC(buffer, VGA_COLOUR_BLACK);
};


//Clears a display buffer to a specified colour
void VGA_ClearC(VGA_Buffer buffer, VGA_Colour c) {
	int	x, y;

	for (x = 0; x < VGA_RES_X; x++) {
		for (y = 0; y < VGA_RES_Y; y++) {
			VGA_PSet(buffer, x, y, c);
		};
	};
};
