`timescale 1ns / 1ps
module data_transfer
(
   iClk_pd,
   iRst_pd,
   oData_pd,
   oGo_pd,
   oEol_pd,
   oEof_pd,

   oClk_vga,
   oAddr_vga,
   iData_vga,
   iEol_vga,
   iEof_vga
);

   ////////////////////////////////////////
   
   input                iClk_pd;
   input                iRst_pd;
   output      [95: 0]  oData_pd;
   output               oGo_pd;
   output               oEol_pd;
   output               oEof_pd;

   output               oClk_vga;
   output      [9: 0]   oAddr_vga;
   input       [7: 0]   iData_vga;
   input                iEol_vga;
   input                iEof_vga;

   ////////////////////////////////////////

   parameter   [3: 0]   S_IDLE = 'd0,
                        S_START = 'd1,
                        S_DONE = 'd2;

   ////////////////////////////////////////

   wire                 wEol;
   wire                 wEof;

   reg      [3: 0]      rCS, rNS;
   reg      [191: 0]    rData;
   reg      [4: 0]      rCounter;
   reg      [10:  0]    rAddr;
   reg                  rGo;
   reg                  rEol, rEol_pulse;
   reg                  rEof, rEof_pulse;

   assign               oClk_vga = iClk_pd;
   assign               oAddr_vga = rAddr;
   assign               oData_pd =  ((rNS == S_START) && (rCounter == 'd12)) ? rData[95: 0]:
                                    ((rNS == S_START) && (rCounter == 'd00)) ? rData[191: 96]:
                                                                               96'h0;
   assign               oGo_pd = rGo;
   assign               oEol_pd = wEol;
   assign               oEof_pd = wEof;
   assign               wEol = (rEol & ~rEol_pulse) ? 1'b1 : 1'b0;
   assign               wEof = (rEof & ~rEof_pulse) ? 1'b1 : 1'b0;

   always @ (*)
      begin
         case (rCS)
            S_IDLE   :  begin
                           if (wEol)
                              begin
                                 rNS = S_START;
                              end
                           else
                              begin
                                 rNS = S_IDLE;
                              end
                        end
            S_START  :  begin
                           if (rAddr == 10'b11_1111_1111)
                              begin
                                 rNS = S_DONE;
                              end
                           else
                              begin
                                 rNS = S_START;
                              end
                        end
            S_DONE   :  begin
                           if (wEol)
                              begin
                                 rNS = S_START;
                              end
                           else
                              begin
                                 rNS = S_DONE;
                              end
                        end
            S_IDLE   :  rNS = S_IDLE;
         endcase
      end

   always @ (posedge iClk_pd)
      begin
         if (iRst_pd)
            begin
               rCS <= S_IDLE;
               rCounter <= 5'b00000;
               rAddr <= 10'b00_0000_0000;
               rData <= 192'h0;
               rGo <= 1'b0;
               rEol <= 1'b0;
               rEol_pulse <= 1'b0;
               rEof <= 1'b0;
               rEof_pulse <= 1'b0;
            end
         else
            begin
               rCS <= rNS;

               rEol <= iEol_vga;
               rEol_pulse <= rEol;
               
               rEof <= iEof_vga;
               rEof_pulse <= rEof;

               if ((rNS == S_DONE) || (rCounter == 'd23))
                  begin
                     rCounter <= 5'b00000;
                  end
               else if (rNS == S_START) 
                  begin
                     rCounter <= rCounter + 1;
                  end

               if (rNS == S_START)
                  begin
                     rAddr <= rAddr + 1;
                  end
               else if (rNS == S_DONE)
                  begin
                     rAddr <= 10'b00_0000_0000;
                  end

               if ((rNS == S_START) && (rCounter == 'd0))
                  begin
                     rData[7: 0] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd1))
                  begin
                     rData[15: 8] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd2))
                  begin
                     rData[23: 16] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd3))
                  begin
                     rData[31: 24] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd4))
                  begin
                     rData[39: 32] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd5))
                  begin
                     rData[47: 40] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd6))
                  begin
                     rData[55: 48] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd7))
                  begin
                     rData[63: 56] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd8))
                  begin
                     rData[71: 64] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd9))
                  begin
                     rData[79: 72] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd10))
                  begin
                     rData[87: 80] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd11))
                  begin
                     rData[95: 88] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd12))
                  begin
                     rData[103: 96] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd13))
                  begin
                     rData[111: 104] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd14))
                  begin
                     rData[119: 112] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd15))
                  begin
                     rData[127: 120] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd16))
                  begin
                     rData[135: 128] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd17))
                  begin
                     rData[143: 136] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd18))
                  begin
                     rData[151: 144] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd19))
                  begin
                     rData[159: 152] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd20))
                  begin
                     rData[167: 160] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd21))
                  begin
                     rData[175: 168] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd22))
                  begin
                     rData[183: 176] <= iData_vga;
                  end
               if ((rNS == S_START) && (rCounter == 'd23))
                  begin
                     rData[191: 184] <= iData_vga;
                  end

               if ((rNS == S_START) && ((rCounter == 'd11) || (rCounter == 'd23)))
                  begin
                     rGo <= 1'b1;
                  end
               else
                  begin
                     rGo <= 1'b0;
                  end
            end
      end

endmodule

