`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:56:46 03/13/2008 
// Design Name: 
// Module Name:    list_mem 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
/*
This module provides storage space for the list of local minimum points. The list of
local minima can contain up to 512 points, which are mirrored over 2 BRAMs. This is
designed to facilitate reading of two different points simultaneously from the same
list.

The 'source' port is from the logic that determines local minima; the port is used
to initialize the list.

The 'list1' and 'list2' ports allow for the clustering hardware to access and rewrite
points to minimize the list.
*/
module list_mem(	source_addr,			//address line for data coming from islocal_min logic
						source_data,			//data line coming from islocal_min logic
						source_clk,				//duh.
						source_en,
						source_wen,
						
						list1_addr,				//point1 address line
						list1_indata,			//input data for writing to point1 memory
						list1_outdata,			//output data for reading from point1 memory
						list1_en,				//duh.
						list1_wen,
						list1_clk,
						list1_ssr,

						list2_addr,				//point1 address line
						list2_indata,			//input data for writing to point1 memory
						list2_outdata,			//output data for reading from point1 memory
						list2_en,				//duh.
						list2_wen,
						list2_clk,
						list2_ssr					
					 );

input	[8:0]		source_addr;			//address line for data coming from islocal_min logic
input	[31:0]	source_data;			//data line coming from islocal_min logic
input				source_clk;				//duh.
input				source_en;
input				source_wen;
	
input	[8:0]		list1_addr;				//point1 address line
input	[31:0]	list1_indata;			//input data for writing to point1 memory
output[31:0]	list1_outdata;			//output data for reading from point1 memory
input				list1_en;				//duh.
input				list1_wen;
input				list1_clk;
input				list1_ssr;

input	[8:0]		list2_addr;				//point1 address line
input	[31:0]	list2_indata;			//input data for writing to point1 memory
output[31:0]	list2_outdata;			//output data for reading from point1 memory
input				list2_en;				//duh.
input				list2_wen;
input				list2_clk;
input				list2_ssr;

wire [31:0]	list1_outdata;
wire [31:0]	list2_outdata;

RAMB16_S36_S36 B1 (
// Input from islocal_min
.ADDRA ( source_addr ),		// I [8:0]
.CLKA  ( source_clk ),     // I
.DIA   ( source_data ),   	// I [31:0]
.DIPA  ( 4'b0 ),     		// I [3:0]
.DOA   (),           		// O [31:0]
.DOPA  (),           		// O [3:0]
.ENA   ( source_en ), 		// I
.SSRA  ( 1'b0 ),     		// I
.WEA   ( source_wen ),    	// I
// in/out from list/cluster
.ADDRB ( list1_addr ),     // I [8:0]
.CLKB  ( list1_clk ),      // I
.DIB   ( list1_indata ),   // I [31:0]
.DIPB  ( 4'b0 ),     		// I [3:0]
.DOB   ( list1_outdata ),  // O [31:0]
.DOPB  (),           		// O [3:0]
.ENB   ( list1_en ),     	// I
.SSRB  ( list1_ssr ),      // I
.WEB   ( list1_wen )       // I
);

RAMB16_S36_S36 B2 (
// Input from islocal_min
.ADDRA ( source_addr ),		// I [8:0]
.CLKA  ( source_clk ),     // I
.DIA   ( source_data ),   	// I [31:0]
.DIPA  ( 4'b0 ),     		// I [3:0]
.DOA   (),           		// O [31:0]
.DOPA  (),           		// O [3:0]
.ENA   ( source_en ), 		// I
.SSRA  ( 1'b0 ),     		// I
.WEA   ( source_wen ),    	// I
// in/out from list/cluster
.ADDRB ( list2_addr ),     // I [8:0]
.CLKB  ( list2_clk ),      // I
.DIB   ( list2_indata ),   // I [31:0]
.DIPB  ( 4'b0 ),     		// I [3:0]
.DOB   ( list2_outdata ),  // O [31:0]
.DOPB  (),           		// O [3:0]
.ENB   ( list2_en ),     	// I
.SSRB  ( list2_ssr ),      // I
.WEB   ( list2_wen )       // I
);

//set READ_FIRST 
/*
defparam B1.WRITE_MODE_A = "NO_CHANGE";
defparam B1.WRITE_MODE_B = "NO_CHANGE";
*/
defparam B2.WRITE_MODE_A = "READ_FIRST";
defparam B2.WRITE_MODE_B = "READ_FIRST";


//Set RAM data contents by 32 byte block.defparam B1.INIT_00 = 'h01020304_05060708_090A0B0C_0D0E0F10_11121314_15161718_191A1B1C_1D1E1F20;
/*
defparam B1.INIT_00 = 'h0003D449_0003D448_00040C46_00040C3E_0004123D_00040A3D_0004123C_00040C3B;
defparam B1.INIT_01 = 'h00000000_00000000_0001E36D_0002676D_0001CB6D_0001C76C_0001C167_0001BF66;
defparam B1.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_04 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_05 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_06 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_07 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_08 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_09 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_0A = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_0B = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_0C = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_0E = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_10 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_12 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_15 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_17 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_18 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_19 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_1A = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_1B = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_1C = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_1D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_1E = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_1F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_20 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_21 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_22 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_23 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_24 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_25 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_26 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_27 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_28 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_29 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_2A = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_2B = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_2C = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_2D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_2E = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_2F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_30 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_31 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_32 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_33 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_34 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_35 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_36 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_37 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_38 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_39 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_3A = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_3B = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_3C = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_3D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_3E = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
//defparam B1.INIT_3F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B1.INIT_3F = 'h0003D449_0003D448_00040C46_00040C3E_0004123D_00040A3D_0004123C_00040C3B;

defparam B2.INIT_00 = 'h0003D449_0003D448_00040C46_00040C3E_0004123D_00040A3D_0004123C_00040C3B;
defparam B2.INIT_01 = 'h00000000_00000000_0001E36D_0002676D_0001CB6D_0001C76C_0001C167_0001BF66;
defparam B2.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_04 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_05 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_06 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_07 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_08 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_09 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_0A = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_0B = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_0C = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_0E = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_10 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_12 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_15 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_17 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_18 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_19 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_1A = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_1B = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_1C = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_1D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_1E = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_1F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_20 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_21 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_22 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_23 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_24 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_25 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_26 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_27 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_28 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_29 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_2A = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_2B = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_2C = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_2D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_2E = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_2F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_30 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_31 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_32 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_33 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_34 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_35 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_36 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_37 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_38 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_39 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_3A = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_3B = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_3C = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_3D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_3E = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
//defparam B2.INIT_3F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam B2.INIT_3F = 'h0003D449_0003D448_00040C46_00040C3E_0004123D_00040A3D_0004123C_00040C3B;
*/


endmodule

