`timescale 1ns / 1ps
//Modified to operate with BRAMs.

module master_FSM02(
					shifter_go,
					cluster_master_go,
					shifter_done,
					pointn,
					pointm,
					SDRAM_addr,					
					clk,			//100MHz
					master_go,
					rst
				);
				
output				shifter_go;
output 				cluster_master_go;
output 	[8:0]	  	pointn;				//pixel position expressed in individual pixels (rows)
output 	[9:0]	   pointm;				//columns
output	[8:0]		SDRAM_addr;	

input					shifter_done;
input 				clk;
input					master_go;
input					rst;

//state register
reg [8:0] state;
reg [9:0] addr_state;
reg [9:0] baseaddr_state;		//holds which line number is currently being processed

//state machine
always@( posedge clk ) begin
	if( rst ) begin
		state <= 0;
		addr_state <= 0;
		baseaddr_state <= 0;
	end
	else begin
		case( state )
			0: begin	//wait state
				if( master_go )
					state <= 1;
			end
			1: state <= 2;			//wait states for combinational logic
			2: state <= 6;
			6: begin
				if( shifter_done )
					state <= 7;
				end
			7: begin
					addr_state <= addr_state + 1;
					if( addr_state > 50 )		//Do 52 blocks and wait for next go signal
						state <= 8;
					else								//repeat for next block
						state <= 1;
				end
			8: begin
				state <= 0;
				baseaddr_state <= baseaddr_state + 1;	//when done, make the base address larger.
				addr_state <= 0;	//set the block address back to zero.
				end		
			default:
				state <= 0;
		endcase
	end
end

//state signal assignments
assign shifter_go 			= (state == 6);
//baseaddr_state = 120 corresponds to 480 lines processed.
assign cluster_master_go 	= ((state == 8)&&(baseaddr_state>=119));


/* The FSM will process a block of 52 x 10 = 520 blocks, which are not interleaved. 

*/
assign pointn 				= baseaddr_state*4;	//rows
assign pointm 				= addr_state*12;						//columns
assign SDRAM_addr 		= baseaddr_state [8:0];


endmodule
