//////////////////////////////////////////////////////////////////////////////
// Filename:          C:\Skule\ECE532\HDL2\EDK_repository\MyProcessorIPLib/drivers/bplbaccess_v1_01_a/src/bplbaccess_selftest.c
// Version:           1.01.a
// Description:       Contains a diagnostic self-test function for the bplbaccess driver
// Date:              Thu Mar 20 17:24:53 2008 (by Create and Import Peripheral Wizard)
//////////////////////////////////////////////////////////////////////////////


/***************************** Include Files *******************************/

#include "bplbaccess.h"

/************************** Constant Definitions ***************************/

#define BPLBACCESS_SELFTEST_BUFSIZE  128 /* Size of buffer (for transfer test) in bytes */

/************************** Variable Definitions ****************************/

static Xuint8 SrcBuffer[BPLBACCESS_SELFTEST_BUFSIZE];   /* Source buffer      */
static Xuint8 DstBuffer[BPLBACCESS_SELFTEST_BUFSIZE];   /* Destination buffer */

/************************** Function Definitions ***************************/

/**
 *
 * Run a self-test on the driver/device. Note this may be a destructive test if
 * resets of the device are performed.
 *
 * If the hardware system is not built correctly, this function may never
 * return to the caller.
 *
 * @param   baseaddr_p is the base address of the BPLBACCESS instance to be worked on.
 *
 * @return
 *
 *    - XST_SUCCESS   if all self-test code passed
 *    - XST_FAILURE   if any self-test code failed
 *
 * @note    Caching must be turned off for this function to work.
 * @note    Self test may fail if data memory and device are not on the same bus.
 *
 */
XStatus BPLBACCESS_SelfTest(void * baseaddr_p)
{
   int     Index;
   Xuint32 baseaddr;
   Xuint8  Reg8Value;
   Xuint16 Reg16Value;
   Xuint32 Reg32Value;
   Xuint64 Reg64Value;

   /*
    * Assert the argument
    */
   XASSERT_NONVOID(baseaddr_p != XNULL);
   baseaddr = (Xuint32) baseaddr_p;

   xil_printf("******************************\n\r");
   xil_printf("* User Peripheral Self Test\n\r");
   xil_printf("******************************\n\n\r");

   /*
    * Reset the device to get it back to its default state and get module identification value
    */
   xil_printf("RST/MIR test...\n\r");
   BPLBACCESS_mReset(baseaddr);
   xil_printf("   - write 0x%08x to software reset register \n\r", IPIF_RESET);
   Reg32Value = BPLBACCESS_mReadMIR(baseaddr);
   if ( Reg32Value == 0x20200308 )
   {
      xil_printf("   - read 0x%08x (expected) from module identification register \n\r", Reg32Value);
      xil_printf("   - RST/MIR write/read passed\n\n\r");
   }
   else
   {
      xil_printf("   - read 0x%08x (unexpected) from module identification register, expect 0x20200308 \n\r", Reg32Value);
      xil_printf("   - RST/MIR write/read failed\n\n\r");
      return XST_FAILURE;
   }

   /*
    * Write to user logic slave module register(s) and read back
    */
   xil_printf("User logic slave module test...\n\r");
   Reg64Value.Upper = 1;
   Reg64Value.Lower = 1;
   xil_printf("   - write 1 to slave register 0 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg0(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg0(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 0 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 1 || Reg64Value.Lower != (Xuint32) 1 )
   {
      xil_printf("   - slave register 0 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 2;
   Reg64Value.Lower = 2;
   xil_printf("   - write 2 to slave register 1 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg1(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg1(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 1 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 2 || Reg64Value.Lower != (Xuint32) 2 )
   {
      xil_printf("   - slave register 1 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 3;
   Reg64Value.Lower = 3;
   xil_printf("   - write 3 to slave register 2 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg2(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg2(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 2 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 3 || Reg64Value.Lower != (Xuint32) 3 )
   {
      xil_printf("   - slave register 2 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 4;
   Reg64Value.Lower = 4;
   xil_printf("   - write 4 to slave register 3 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg3(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg3(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 3 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 4 || Reg64Value.Lower != (Xuint32) 4 )
   {
      xil_printf("   - slave register 3 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 5;
   Reg64Value.Lower = 5;
   xil_printf("   - write 5 to slave register 4 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg4(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg4(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 4 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 5 || Reg64Value.Lower != (Xuint32) 5 )
   {
      xil_printf("   - slave register 4 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 6;
   Reg64Value.Lower = 6;
   xil_printf("   - write 6 to slave register 5 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg5(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg5(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 5 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 6 || Reg64Value.Lower != (Xuint32) 6 )
   {
      xil_printf("   - slave register 5 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 7;
   Reg64Value.Lower = 7;
   xil_printf("   - write 7 to slave register 6 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg6(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg6(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 6 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 7 || Reg64Value.Lower != (Xuint32) 7 )
   {
      xil_printf("   - slave register 6 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 8;
   Reg64Value.Lower = 8;
   xil_printf("   - write 8 to slave register 7 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg7(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg7(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 7 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 8 || Reg64Value.Lower != (Xuint32) 8 )
   {
      xil_printf("   - slave register 7 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 9;
   Reg64Value.Lower = 9;
   xil_printf("   - write 9 to slave register 8 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg8(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg8(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 8 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 9 || Reg64Value.Lower != (Xuint32) 9 )
   {
      xil_printf("   - slave register 8 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 10;
   Reg64Value.Lower = 10;
   xil_printf("   - write 10 to slave register 9 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg9(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg9(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 9 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 10 || Reg64Value.Lower != (Xuint32) 10 )
   {
      xil_printf("   - slave register 9 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 11;
   Reg64Value.Lower = 11;
   xil_printf("   - write 11 to slave register 10 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg10(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg10(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 10 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 11 || Reg64Value.Lower != (Xuint32) 11 )
   {
      xil_printf("   - slave register 10 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 12;
   Reg64Value.Lower = 12;
   xil_printf("   - write 12 to slave register 11 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg11(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg11(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 11 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 12 || Reg64Value.Lower != (Xuint32) 12 )
   {
      xil_printf("   - slave register 11 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 13;
   Reg64Value.Lower = 13;
   xil_printf("   - write 13 to slave register 12 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg12(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg12(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 12 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 13 || Reg64Value.Lower != (Xuint32) 13 )
   {
      xil_printf("   - slave register 12 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 14;
   Reg64Value.Lower = 14;
   xil_printf("   - write 14 to slave register 13 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg13(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg13(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 13 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 14 || Reg64Value.Lower != (Xuint32) 14 )
   {
      xil_printf("   - slave register 13 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 15;
   Reg64Value.Lower = 15;
   xil_printf("   - write 15 to slave register 14 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg14(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg14(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 14 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 15 || Reg64Value.Lower != (Xuint32) 15 )
   {
      xil_printf("   - slave register 14 write/read failed\n\r");
      return XST_FAILURE;
   }
   Reg64Value.Upper = 16;
   Reg64Value.Lower = 16;
   xil_printf("   - write 16 to slave register 15 upper and lower portion\n\r");
   BPLBACCESS_WriteSlaveReg15(baseaddr_p, &Reg64Value);
   BPLBACCESS_ReadSlaveReg15(baseaddr_p, &Reg64Value);
   xil_printf("   - read %d, %d from register 15 upper and lower portion \n\r", Reg64Value.Upper, Reg64Value.Lower);
   if ( Reg64Value.Upper != (Xuint32) 16 || Reg64Value.Lower != (Xuint32) 16 )
   {
      xil_printf("   - slave register 15 write/read failed\n\r");
      return XST_FAILURE;
   }
   xil_printf("   - slave register write/read passed\n\n\r");

   /*
    * Write to write packet FIFO and read back from read packet FIFO
    */
   xil_printf("Packet FIFO test...\n\r");
   xil_printf("   - reset write packet FIFO to initial state \n\r");
   BPLBACCESS_mResetWriteFIFO(baseaddr);
   xil_printf("   - reset read packet FIFO to initial state \n\r");
   BPLBACCESS_mResetReadFIFO(baseaddr);
   xil_printf("   - push data to write packet FIFO \n\r");
   for ( Index = 0; Index < 4; Index++ )
   {
      Reg64Value.Upper = Index;
      Reg64Value.Lower = Index;
      xil_printf("     0x%08x_%08x \n\r", Reg64Value.Upper, Reg64Value.Lower);
      BPLBACCESS_WriteToFIFO(baseaddr_p, &Reg64Value);
   }
   xil_printf("   - write packet FIFO is %s \n\r", ( BPLBACCESS_mWriteFIFOFull(baseaddr) ? "full" : "not full" ));
   xil_printf("   - pop data out from read packet FIFO \n\r");
   for ( Index = 0; Index < 4; Index++ )
   {
      BPLBACCESS_ReadFromFIFO(baseaddr_p, &Reg64Value);
      xil_printf("     0x%08x_%08x \n\r", Reg64Value.Upper, Reg64Value.Lower);
      if ( Reg64Value.Upper != (Xuint32) Index || Reg64Value.Lower != (Xuint32) Index )
      {
         xil_printf("   - unexpected value read from read packet FIFO \n\r");
         xil_printf("   - write/read packet FIFO failed \n\r");
         BPLBACCESS_mResetWriteFIFO(baseaddr);
         BPLBACCESS_mResetReadFIFO(baseaddr);
         return XST_FAILURE;
      }
   }
   xil_printf("   - read packet FIFO is %s \n\r", ( BPLBACCESS_mReadFIFOEmpty(baseaddr) ? "empty" : "not empty" ));
   BPLBACCESS_mResetWriteFIFO(baseaddr);
   BPLBACCESS_mResetReadFIFO(baseaddr);
   xil_printf("   - write/read packet FIFO passed \n\n\r");

   /*
    * Enable all possible interrupts and clear interrupt status register(s)
    */
   xil_printf("Interrupt controller test...\n\r");
   Reg32Value = BPLBACCESS_mReadReg(baseaddr, BPLBACCESS_INTR_ISR_OFFSET);
   xil_printf("   - IP (user logic) interrupt status : 0x%08x \n\r", Reg32Value);
   xil_printf("   - clear IP (user logic) interrupt status register\n\r");
   BPLBACCESS_mWriteReg(baseaddr, BPLBACCESS_INTR_ISR_OFFSET, Reg32Value);
   Reg32Value = BPLBACCESS_mReadReg(baseaddr, BPLBACCESS_INTR_DISR_OFFSET);
   xil_printf("   - Device (peripheral) interrupt status : 0x%08x \n\r", Reg32Value);
   xil_printf("   - clear Device (peripheral) interrupt status register\n\r");
   BPLBACCESS_mWriteReg(baseaddr, BPLBACCESS_INTR_DISR_OFFSET, Reg32Value);
   xil_printf("   - enable all possible interrupt(s)\n\r");
   BPLBACCESS_EnableInterrupt(baseaddr_p);
   xil_printf("   - write/read interrupt register passed \n\n\r");

   /*
    * Setup DMA channel 0 to receive data from source buffer to local FIFO and
    * DMA channel 1 to send data from local FIFO to destination buffer
    */
   xil_printf("Simple DMA test...\n\r");
   xil_printf("   - source buffer address is 0x%08x \n\r", SrcBuffer);
   xil_printf("   - destination buffer address is 0x%08x \n\r", DstBuffer);
   xil_printf("   - initialize the source buffer bytes with a pattern \n\r");
   xil_printf("   - initialize the destination buffer bytes to zero \n\r");
   for ( Index = 0; Index < BPLBACCESS_SELFTEST_BUFSIZE; Index++ )
   {
      SrcBuffer[Index] = Index;
      DstBuffer[Index] = 0;
   }

   xil_printf("   - \n\r");
   xil_printf("   - reset DMA channel 0 to initial state \n\r");
   BPLBACCESS_mResetDMA0(baseaddr);
   xil_printf("   - setup DMA channel 0 to transfer data from remote source to local FIFO \n\r");
   BPLBACCESS_mSetDMA0Control(baseaddr, DMA_SINC_MASK | DMA_DLOCAL_MASK);
   xil_printf("   - start DMA channel 0 to receive bytes from the source buffer \n\r");
   BPLBACCESS_DMA0Transfer(baseaddr, (Xuint32) SrcBuffer, baseaddr+BPLBACCESS_WRFIFO_DATA_OFFSET, BPLBACCESS_SELFTEST_BUFSIZE);
   while ( ! BPLBACCESS_mDMA0Done(baseaddr) ) {}
   xil_printf("   - DMA channel 0 transfer completed \n\r");
   if ( BPLBACCESS_mDMA0Error(baseaddr) )
   {
      xil_printf("   - DMA channel 0 transfer error \n\r");
      xil_printf("   - DMA failed \n\r");
      return XST_FAILURE;
   }
   xil_printf("   - reset DMA channel 1 to initial state \n\r");
   BPLBACCESS_mResetDMA1(baseaddr);
   xil_printf("   - setup DMA channel 1 to transfer data from local FIFO to remote destination \n\r");
   BPLBACCESS_mSetDMA1Control(baseaddr, DMA_DINC_MASK | DMA_SLOCAL_MASK);
   xil_printf("   - start DMA channel 1 to send bytes to the destination buffer \n\r");
   BPLBACCESS_DMA1Transfer(baseaddr, baseaddr+BPLBACCESS_RDFIFO_DATA_OFFSET, (Xuint32) DstBuffer, BPLBACCESS_SELFTEST_BUFSIZE);
   while ( ! BPLBACCESS_mDMA1Done(baseaddr) ) {}
   xil_printf("   - DMA channel 1 transfer completed \n\r");
   if ( BPLBACCESS_mDMA1Error(baseaddr) )
   {
      xil_printf("   - DMA channel 1 transfer error \n\r");
      xil_printf("   - DMA failed \n\r");
      return XST_FAILURE;
   }
   for ( Index = 0; Index < BPLBACCESS_SELFTEST_BUFSIZE; Index++ )
   {
      if ( DstBuffer[Index] != SrcBuffer[Index] )
      {
         xil_printf("   - destination buffer's contents are different from the source buffer \n\r");
         xil_printf("   - DMA failed \n\r");
         return XST_FAILURE;
      }
   }
   xil_printf("   - destination buffer's contents are same as the source buffer \n\r");
   BPLBACCESS_mResetDMA0(baseaddr);
   BPLBACCESS_mResetDMA1(baseaddr);
   xil_printf("   - Simple DMA transfer passed \n\n\r");

   return XST_SUCCESS;
}
