`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    12:32:39 03/04/2008 
// Design Name: 
// Module Name:    bram_buffer 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

/* This module holds a RAM buffer for use in the hardware image processing core.
It operates according to the notes made on the tablet. 

Update, 1:23AM, Thursday March 27, 2008
This core now works by connecting directly to the video capture core. Input is 96 bits, with a new addressing scheme.

*/
module bram_buffer2(			//BUFFER CONTROL SIGNALS FOR VIDEO CAPTURE CORE BRAM PORT
							HCAP_data,			//I [95:0], input data from PLB SDRAM reader module
							HCAP_wen,			//I straight-through to BRAM
							//HCAP_line_sel,		//I [3:0] picks one of 11 line numbers
							//HCAP_pix_addr,		//I [5:0] picks block of 12 pixels in a given line							
							HCAP_clk,			//I clock for HCAP port of BRAM
							line_pulse,			//I at the end of every line
							frame_pulse,		//I at the end of every frame
							//BUFFER CONTROL SIGNALS FOR PROCESSING CORE BRAM PORT
							//M_matrix_data,		//O [959:0] Output data array, probably 2D, byte-wide (3D)
							oline0, 
							oline1, 
							oline2, 
							oline3, 
							oline4, 
							oline5, 
							oline6,
							oline7,
							oline8,
							oline9, 
							
							M_proc_ready,		//O one cycle pulse to say that a block of 10 lines is ready for processing at online0-10.
							M_matrix_addr,		//I [8:0] Address
							M_matrix_clk,		//I
							rst					//global reset signal
				   );
	
	//I/O Declarations
	input 		[95:0] 		HCAP_data;
	input							HCAP_wen;
	//input		[3:0]			HCAP_line_sel;	//selects which line to write to. VALID INPUT RANGE IS 0-11 in decimal
	//input		[5:0]			HCAP_pix_addr;	//pixel addresses in groups of 96 bits (12 pixels.) This means that addr 0 accesses pixels 0-11,
														//address 1 accesses 12-23, and so on.	
	input							HCAP_clk;
	input 						line_pulse;		
	input							frame_pulse;
	
	//output		[959:0]		M_matrix_data;	
	//line outputs
	output		[95:0]		oline0;
	output		[95:0]		oline1;
	output		[95:0]		oline2;
	output		[95:0]		oline3;
	output		[95:0]		oline4;
	output		[95:0]		oline5;
	output		[95:0]		oline6;
	output		[95:0]		oline7;
	output		[95:0]		oline8;
	output		[95:0]		oline9;
	                      
	output						M_proc_ready;
	input			[8:0]			M_matrix_addr;
	input							M_matrix_clk;
	input 						rst;
	
	wire 			[31:0]		HCAP_data_LSB;	//lowest bits
	wire			[31:0]		HCAP_data_ISB;	//intermediate (middle) bits
	wire 			[31:0]		HCAP_data_MSB;	//highest bits

	//M port output wires
	wire		[31:0]		BRAM_DOB_00;
	wire		[31:0]		BRAM_DOB_01;
	wire		[31:0]		BRAM_DOB_02;
	wire		[31:0]		BRAM_DOB_03;
	wire		[31:0]		BRAM_DOB_04;
	wire		[31:0]		BRAM_DOB_05;
	wire		[31:0]		BRAM_DOB_06;
	wire		[31:0]		BRAM_DOB_07;
	wire		[31:0]		BRAM_DOB_08;
	wire		[31:0]		BRAM_DOB_09;
	wire		[31:0]		BRAM_DOB_10;
	wire		[31:0]		BRAM_DOB_11;
	wire		[31:0]		BRAM_DOB_12;
	wire		[31:0]		BRAM_DOB_13;
	wire		[31:0]		BRAM_DOB_14;
	wire		[31:0]		BRAM_DOB_15;
	wire		[31:0]		BRAM_DOB_16;
	wire		[31:0]		BRAM_DOB_17;
	wire		[31:0]		BRAM_DOB_18;
	wire		[31:0]		BRAM_DOB_19;
	wire		[31:0]		BRAM_DOB_20;
	wire		[31:0]		BRAM_DOB_21;
	wire		[31:0]		BRAM_DOB_22;
	wire		[31:0]		BRAM_DOB_23;
	wire		[31:0]		BRAM_DOB_24;
	wire		[31:0]		BRAM_DOB_25;
	wire		[31:0]		BRAM_DOB_26;
	wire		[31:0]		BRAM_DOB_27;
	wire		[31:0]		BRAM_DOB_28;
	wire		[31:0]		BRAM_DOB_29;
	wire		[31:0]		BRAM_DOB_30;
	wire		[31:0]		BRAM_DOB_31;
	wire		[31:0]		BRAM_DOB_32;
	wire		[31:0]		BRAM_DOB_33;
	wire		[31:0]		BRAM_DOB_34;
	wire		[31:0]		BRAM_DOB_35;
	wire		[31:0]		BRAM_DOB_36;
	wire		[31:0]		BRAM_DOB_37;
	wire		[31:0]		BRAM_DOB_38;
	wire		[31:0]		BRAM_DOB_39;
	wire		[31:0]		BRAM_DOB_40;
	wire		[31:0]		BRAM_DOB_41;	

	wire		[95:0]		line0 ;
	wire		[95:0]		line1 ;
	wire		[95:0]		line2 ;
	wire		[95:0]		line3 ;
	wire		[95:0]		line4 ;
	wire		[95:0]		line5 ;
	wire		[95:0]		line6 ;
	wire		[95:0]		line7 ;
	wire		[95:0]		line8 ;
	wire		[95:0]		line9 ;
	wire		[95:0]		line10 ;
	wire		[95:0]		line11 ;
	wire		[95:0]		line12 ;
	wire		[95:0]		line13 ;	
	
	
	assign HCAP_data_LSB = HCAP_data [31:0];
	assign HCAP_data_ISB = HCAP_data [63:32];
	assign HCAP_data_MSB = HCAP_data [95:64];
	
	//dummy registers for mux outputs
	reg			[95:0]		oline0;
	reg			[95:0]		oline1;
	reg			[95:0]		oline2;
	reg			[95:0]		oline3;
	reg			[95:0]		oline4;
	reg			[95:0]		oline5;
	reg			[95:0]		oline6;
	reg			[95:0]		oline7;
	reg			[95:0]		oline8;
	reg			[95:0]		oline9;

	reg 		[8:0] 		HCAP_addr;		//state register to track address to write to.
	reg 		[3:0] 		w_line_sel;		//keeps track of which set of four lines is being written to.
	reg			[1:0]		state;
	reg			[3:0]		init_count;		//init counter to fill up first 10 rows before processing
	reg			[8:0]		line_count;		//keeps track of total number of lines already processed	
	reg						set;
	reg		[4:0]			HCAP_line_sel;

	//simple state logic
	always@( posedge HCAP_clk ) begin
		if( frame_pulse || rst ) begin		//on reset. Duh.
			HCAP_addr = 0;
			w_line_sel = 0;				
			state = 0;
			init_count = 0;
			line_count = 0;
			set = 0;
		end
		else begin
			if( HCAP_wen ) begin
				HCAP_addr = HCAP_addr + 1;
				
			end
			if( line_pulse ) begin
				HCAP_addr = 0;
				HCAP_line_sel = HCAP_line_sel +1;
				if( HCAP_line_sel >= 14 ) begin
					HCAP_line_sel = 0;
				end
			end
			case( state )
				0: begin
					if( line_pulse ) begin
						state = 1;
					end
				end
				1: begin						//count 11 lines initially, to start the read off.
					if( line_pulse) begin
						init_count = init_count + 1;
						//line_count = line_count + 1;
						if( init_count >= 6 ) begin
							state = 2;
							line_count = line_count + 1;
						end
					end
				end
				2: begin
					if( line_pulse ) begin
						line_count = line_count + 1;
						set = 0;
					end
					if( (line_count [1:0] == 2'b0) && (set== 0) ) begin
						state = 3;
					end
				end
				3: begin
					set = 1;
					if( w_line_sel >= 6 )
						w_line_sel = 0;
					else
						w_line_sel = w_line_sel + 1;
					state = 2;
				end
				default: begin
					 state = 0;
				end
			endcase
		end
	end

	//combinatorial state assigmments
	assign 	M_proc_ready = (state == 3);
	
	
	//fun parameters!
	//how many block RAMs there are.
	//parameter NUM_BRAMS = 33;
	//how many groups the BRAMs are divided into for writing from SDRAM. (how many lines)
	parameter NUM_BLOCKS = 14;

	reg BRAM_SSRB = 0;

	//INPUT DATA MUX
	//Sets appropriate WEA and ENA signals by keeping a one-hot array of enable signals.
	reg [NUM_BLOCKS-1:0] BRAM_WEA_sel;
	reg [NUM_BLOCKS-1:0] BRAM_ENA_sel;

	//One-hot the write enable signals so that only one block is written to at a time.
	always@( posedge HCAP_clk )
	begin 
		BRAM_WEA_sel = HCAP_wen;
		BRAM_ENA_sel = 1'b1 << HCAP_line_sel;	
	end

RAMB16_S36_S36 RGB_BRAM_00 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[0] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[0] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_00 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_01 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[0] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[0] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_01 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_02 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[0] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[0] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_02 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_03 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[1] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[1] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_03 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_04 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[1] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[1] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_04 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_05 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[1] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[1] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_05 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_06 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[2] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[2] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_06 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_07 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[2] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[2] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_07 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_08 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[2] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[2] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_08 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_09 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[3] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[3] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_09 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_10 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[3] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[3] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_10 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_11 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[3] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[3] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_11 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_12 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[4] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[4] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_12 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_13 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[4] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[4] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_13 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_14 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[4] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[4] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_14 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_15 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[5] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[5] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_15 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_16 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[5] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[5] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_16 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_17 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[5] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[5] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_17 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_18 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[6] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[6] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_18 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_19 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[6] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[6] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_19 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_20 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[6] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[6] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_20 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_21 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[7] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[7] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_21 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_22 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[7] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[7] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_22 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_23 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[7] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[7] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_23 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_24 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[8] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[8] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_24 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_25 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[8] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[8] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_25 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_26 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[8] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[8] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_26 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_27 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[9] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[9] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_27 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_28 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[9] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[9] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_28 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_29 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[9] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[9] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_29 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_30 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[10] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[10] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_30 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_31 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[10] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[10] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_31 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_32 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[10] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[10] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_32 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_33 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[11] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[11] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_33 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_34 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[11] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[11] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_34 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_35 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[11] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[11] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_35 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_36 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[12] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[12] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_36 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_37 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[12] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[12] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_37 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_38 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[12] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[12] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_38 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_39 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_MSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[13] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[13] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_39 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_40 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_ISB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[13] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[13] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_40 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);


RAMB16_S36_S36 RGB_BRAM_41 (
// Input from RAM (PORT A)
.ADDRA ( HCAP_addr ),       // I [8:0]
.CLKA  ( HCAP_clk ),        // I
.DIA   ( HCAP_data_LSB ),   // I [31:0]
.DIPA  ( 4'b0 ),           // I [3:0]
.DOA   (),                   // O [31:0]
.DOPA  (),                   // O [3:0]
.ENA   ( BRAM_ENA_sel[13] ), // I
.SSRA  ( 1'b0 ),         	// I
.WEA   ( BRAM_WEA_sel[13] ),    // I
// Buffer output (PORT B)
.ADDRB ( M_matrix_addr ),       // I [8:0]
.CLKB  ( M_matrix_clk ),        // I
.DIB   ( 32'b0 ),          // I [31:0]
.DIPB  ( 4'b0 ),           // I [3:0]
.DOB   ( BRAM_DOB_41 ),     // O [31:0]
.DOPB  (),                   // O [3:0]
.ENB   ( 1'b1 ),           // I
.SSRB  ( BRAM_SSRB ),        // I
.WEB   ( 1'b0 )          	// I
);

//OUTPUT BUS DATA
assign line0  = { BRAM_DOB_00, BRAM_DOB_01, BRAM_DOB_02 };
assign line1  = { BRAM_DOB_03, BRAM_DOB_04, BRAM_DOB_05 };
assign line2  = { BRAM_DOB_06, BRAM_DOB_07, BRAM_DOB_08 };
assign line3  = { BRAM_DOB_09, BRAM_DOB_10, BRAM_DOB_11 };
assign line4  = { BRAM_DOB_12, BRAM_DOB_13, BRAM_DOB_14 };
assign line5  = { BRAM_DOB_15, BRAM_DOB_16, BRAM_DOB_17 };
assign line6  = { BRAM_DOB_18, BRAM_DOB_19, BRAM_DOB_20 };
assign line7  = { BRAM_DOB_21, BRAM_DOB_22, BRAM_DOB_23 };
assign line8  = { BRAM_DOB_24, BRAM_DOB_25, BRAM_DOB_26 };
assign line9  = { BRAM_DOB_27, BRAM_DOB_28, BRAM_DOB_29 };
assign line10 = { BRAM_DOB_30, BRAM_DOB_31, BRAM_DOB_32 };
assign line11 = { BRAM_DOB_33, BRAM_DOB_34, BRAM_DOB_35 };
assign line12 = { BRAM_DOB_36, BRAM_DOB_37, BRAM_DOB_38 };
assign line13 = { BRAM_DOB_39, BRAM_DOB_40, BRAM_DOB_41 };

//mux the stuff so the outputs are always not being written to.
	always@( * ) begin
	case( w_line_sel )
		0: begin		//writing to lines 10-13
			oline0 = line0;
			oline1 = line1;
			oline2 = line2; 
			oline3 = line3;
			oline4 = line4;
			oline5 = line5;
			oline6 = line6; 
			oline7 = line7;
			oline8 = line8; 
			oline9 = line9;
		end		
		1: begin		//writing to lines 0-3
			oline0 = line4;
			oline1 = line5;
			oline2 = line6; 
			oline3 = line7;
			oline4 = line8;
			oline5 = line9;
			oline6 = line10; 
			oline7 = line11;
			oline8 = line12; 
			oline9 = line13;		
		end
		2: begin		//writing to lines 4-7
			oline0 = line8;
			oline1 = line9;
			oline2 = line10; 
			oline3 = line11;
			oline4 = line12;
			oline5 = line13;
			oline6 = line0; 
			oline7 = line1;
			oline8 = line2; 
			oline9 = line3;
		end
		3: begin		//writing to lines 8-11
			oline0 = line12;
			oline1 = line13;
			oline2 = line0; 
			oline3 = line1;
			oline4 = line2;
			oline5 = line3;
			oline6 = line4; 
			oline7 = line5;
			oline8 = line6; 
			oline9 = line7;
		end
		4: begin		//writing to lines 12-13,0,1
			oline0 = line2;
			oline1 = line3;
			oline2 = line4; 
			oline3 = line5;
			oline4 = line6;
			oline5 = line7;
			oline6 = line8; 
			oline7 = line9;
			oline8 = line10; 
			oline9 = line11;
		end
		5: begin		//writing to lines 2-5
			oline0 = line6;
			oline1 = line7;
			oline2 = line8; 
			oline3 = line9;
			oline4 = line10;
			oline5 = line11;
			oline6 = line12; 
			oline7 = line13;
			oline8 = line0; 
			oline9 = line1;
		end
		6: begin		//writing to lines 6-9
			oline0 = line10;
			oline1 = line11;
			oline2 = line12; 
			oline3 = line13;
			oline4 = line0;
			oline5 = line1;
			oline6 = line2; 
			oline7 = line3;
			oline8 = line4; 
			oline9 = line5;
		end
		default: begin
			oline0 = line0;
			oline1 = line1;
			oline2 = line2; 
			oline3 = line3;
			oline4 = line4;
			oline5 = line5;
			oline6 = line6; 
			oline7 = line7;
			oline8 = line8; 
			oline9 = line9;
		end
	endcase
	end



defparam RGB_BRAM_00.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_00.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_00.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_00.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_00.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_00.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_00.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_00.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_00.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_00.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_00.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_00.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_00.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_00.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_00.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_00.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_00.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_00.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_00.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_00.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_00.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_00.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_00.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_00.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_00.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_00.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_00.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_00.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_00.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_00.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_00.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_00.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_00.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_00.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_00.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_00.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_00.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_00.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_00.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_00.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_00.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_01.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_01.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_01.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_01.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_01.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_01.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_01.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_01.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_01.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_01.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_01.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_01.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_01.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_01.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_01.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_01.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_01.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_01.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_01.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_01.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_01.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_01.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_01.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_01.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_01.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_01.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_01.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_01.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_01.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_01.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_01.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_01.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_01.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_01.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_01.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_01.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_01.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_01.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_01.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_01.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_01.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_02.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_02.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_02.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_02.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_02.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_02.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_02.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_02.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_02.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_02.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_02.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_02.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_02.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_02.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_02.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_02.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_02.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_02.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_02.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_02.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_02.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_02.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_02.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_02.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_02.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_02.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_02.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_02.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_02.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_02.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_02.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_02.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_02.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_02.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_02.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_02.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_02.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_02.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_02.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_02.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_02.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_03.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_03.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_03.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_03.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_03.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_03.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_03.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_03.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_03.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_03.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_03.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_03.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_03.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_03.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_03.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_03.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_03.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_03.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_03.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_03.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_03.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_03.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_03.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_03.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_03.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_03.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_03.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_03.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_03.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_03.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_03.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_03.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_03.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_03.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_03.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_03.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_03.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_03.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_03.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_03.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_03.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_04.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_04.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_04.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_04.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_04.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_04.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_04.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_04.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_04.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_04.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_04.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_04.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_04.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_04.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_04.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_04.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_04.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_04.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_04.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_04.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_04.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_04.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_04.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_04.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_04.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_04.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_04.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_04.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_04.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_04.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_04.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_04.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_04.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_04.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_04.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_04.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_04.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_04.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_04.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_04.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_04.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_05.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_05.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_05.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_05.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_05.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_05.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_05.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_05.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_05.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_05.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_05.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_05.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_05.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_05.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_05.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_05.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_05.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_05.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_05.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_05.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_05.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_05.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_05.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_05.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_05.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_05.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_05.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_05.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_05.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_05.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_05.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_05.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_05.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_05.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_05.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_05.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_05.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_05.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_05.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_05.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_05.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_06.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_06.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_06.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_06.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_06.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_06.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_06.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_06.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_06.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_06.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_06.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_06.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_06.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_06.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_06.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_06.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_06.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_06.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_06.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_06.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_06.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_06.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_06.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_06.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_06.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_06.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_06.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_06.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_06.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_06.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_06.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_06.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_06.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_06.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_06.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_06.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_06.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_06.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_06.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_06.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_06.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_07.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_07.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_07.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_07.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_07.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_07.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_07.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_07.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_07.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_07.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_07.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_07.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_07.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_07.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_07.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_07.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_07.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_07.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_07.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_07.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_07.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_07.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_07.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_07.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_07.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_07.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_07.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_07.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_07.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_07.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_07.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_07.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_07.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_07.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_07.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_07.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_07.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_07.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_07.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_07.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_07.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_08.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_08.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_08.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_08.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_08.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_08.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_08.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_08.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_08.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_08.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_08.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_08.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_08.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_08.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_08.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_08.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_08.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_08.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_08.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_08.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_08.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_08.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_08.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_08.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_08.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_08.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_08.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_08.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_08.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_08.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_08.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_08.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_08.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_08.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_08.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_08.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_08.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_08.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_08.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_08.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_08.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_09.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_09.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_09.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_09.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_09.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_09.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_09.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_09.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_09.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_09.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_09.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_09.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_09.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_09.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_09.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_09.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_09.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_09.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_09.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_09.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_09.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_09.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_09.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_09.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_09.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_09.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_09.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_09.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_09.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_09.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_09.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_09.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_09.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_09.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_09.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_09.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_09.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_09.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_09.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_09.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_09.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_10.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_10.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_10.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_10.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_10.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_10.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_10.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_10.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_10.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_10.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_10.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_10.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_10.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_10.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_10.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_10.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_10.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_10.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_10.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_10.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_10.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_10.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_10.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_10.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_10.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_10.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_10.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_10.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_10.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_10.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_10.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_10.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_10.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_10.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_10.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_10.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_10.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_10.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_10.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_10.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_10.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_11.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_11.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_11.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_11.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_11.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_11.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_11.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_11.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_11.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_11.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_11.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_11.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_11.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_11.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_11.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_11.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_11.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_11.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_11.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_11.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_11.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_11.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_11.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_11.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_11.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_11.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_11.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_11.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_11.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_11.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_11.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_11.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_11.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_11.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_11.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_11.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_11.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_11.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_11.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_11.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_11.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_12.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_12.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_12.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_12.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_12.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_12.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_12.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_12.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_12.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_12.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_12.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_12.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_12.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_12.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_12.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_12.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_12.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_12.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_12.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_12.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_12.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_12.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_12.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_12.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_12.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_12.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_12.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_12.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_12.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_12.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_12.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_12.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_12.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_12.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_12.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_12.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_12.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_12.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_12.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_12.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_12.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_13.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_13.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_13.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_13.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_13.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_13.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_13.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_13.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_13.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_13.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_13.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_13.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_13.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_13.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_13.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_13.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_13.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_13.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_13.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_13.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_13.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_13.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_13.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_13.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_13.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_13.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_13.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_13.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_13.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_13.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_13.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_13.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_13.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_13.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_13.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_13.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_13.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_13.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_13.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_13.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_13.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_14.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_14.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_14.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_14.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_14.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_14.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_14.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_14.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_14.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_14.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_14.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_14.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_14.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_14.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_14.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_14.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_14.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_14.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_14.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_14.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_14.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_14.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_14.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_14.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_14.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_14.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_14.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_14.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_14.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_14.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_14.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_14.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_14.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_14.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_14.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_14.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_14.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_14.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_14.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_14.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_14.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_15.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_15.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_15.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_15.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_15.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_15.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_15.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_15.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_15.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_15.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_15.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_15.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_15.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_15.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_15.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_15.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_15.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_15.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_15.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_15.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_15.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_15.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_15.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_15.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_15.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_15.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_15.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_15.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_15.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_15.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_15.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_15.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_15.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_15.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_15.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_15.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_15.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_15.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_15.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_15.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_15.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_16.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_16.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_16.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_16.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_16.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_16.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_16.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_16.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_16.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_16.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_16.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_16.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_16.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_16.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_16.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_16.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_16.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_16.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_16.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_16.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_16.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_16.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_16.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_16.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_16.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_16.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_16.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_16.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_16.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_16.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_16.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_16.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_16.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_16.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_16.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_16.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_16.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_16.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_16.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_16.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_16.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_17.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_17.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_17.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_17.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_17.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_17.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_17.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_17.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_17.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_17.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_17.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_17.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_17.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_17.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_17.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_17.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_17.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_17.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_17.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_17.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_17.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_17.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_17.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_17.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_17.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_17.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_17.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_17.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_17.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_17.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_17.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_17.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_17.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_17.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_17.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_17.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_17.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_17.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_17.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_17.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_17.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_18.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_18.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_18.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_18.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_18.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_18.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_18.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_18.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_18.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_18.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_18.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_18.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_18.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_18.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_18.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_18.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_18.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_18.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_18.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_18.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_18.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_18.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_18.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_18.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_18.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_18.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_18.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_18.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_18.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_18.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_18.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_18.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_18.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_18.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_18.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_18.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_18.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_18.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_18.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_18.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_18.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_19.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_19.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_19.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_19.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_19.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_19.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_19.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_19.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_19.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_19.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_19.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_19.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_19.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_19.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_19.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_19.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_19.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_19.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_19.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_19.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_19.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_19.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_19.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_19.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_19.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_19.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_19.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_19.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_19.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_19.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_19.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_19.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_19.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_19.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_19.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_19.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_19.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_19.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_19.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_19.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_19.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_20.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_20.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_20.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_20.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_20.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_20.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_20.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_20.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_20.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_20.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_20.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_20.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_20.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_20.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_20.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_20.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_20.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_20.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_20.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_20.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_20.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_20.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_20.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_20.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_20.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_20.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_20.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_20.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_20.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_20.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_20.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_20.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_20.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_20.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_20.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_20.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_20.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_20.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_20.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_20.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_20.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_21.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_21.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_21.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_21.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_21.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_21.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_21.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_21.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_21.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_21.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_21.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_21.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_21.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_21.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_21.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_21.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_21.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_21.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_21.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_21.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_21.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_21.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_21.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_21.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_21.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_21.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_21.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_21.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_21.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_21.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_21.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_21.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_21.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_21.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_21.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_21.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_21.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_21.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_21.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_21.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_21.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_22.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_22.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_22.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_22.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_22.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_22.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_22.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_22.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_22.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_22.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_22.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_22.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_22.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_22.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_22.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_22.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_22.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_22.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_22.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_22.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_22.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_22.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_22.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_22.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_22.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_22.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_22.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_22.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_22.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_22.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_22.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_22.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_22.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_22.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_22.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_22.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_22.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_22.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_22.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_22.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_22.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_23.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_23.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_23.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_23.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_23.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_23.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_23.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_23.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_23.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_23.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_23.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_23.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_23.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_23.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_23.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_23.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_23.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_23.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_23.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_23.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_23.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_23.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_23.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_23.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_23.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_23.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_23.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_23.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_23.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_23.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_23.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_23.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_23.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_23.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_23.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_23.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_23.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_23.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_23.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_23.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_23.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_24.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_24.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_24.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_24.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_24.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_24.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_24.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_24.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_24.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_24.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_24.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_24.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_24.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_24.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_24.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_24.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_24.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_24.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_24.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_24.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_24.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_24.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_24.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_24.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_24.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_24.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_24.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_24.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_24.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_24.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_24.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_24.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_24.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_24.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_24.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_24.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_24.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_24.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_24.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_24.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_24.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_25.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_25.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_25.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_25.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_25.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_25.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_25.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_25.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_25.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_25.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_25.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_25.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_25.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_25.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_25.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_25.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_25.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_25.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_25.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_25.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_25.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_25.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_25.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_25.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_25.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_25.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_25.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_25.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_25.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_25.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_25.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_25.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_25.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_25.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_25.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_25.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_25.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_25.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_25.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_25.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_25.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_26.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_26.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_26.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_26.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_26.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_26.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_26.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_26.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_26.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_26.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_26.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_26.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_26.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_26.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_26.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_26.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_26.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_26.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_26.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_26.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_26.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_26.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_26.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_26.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_26.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_26.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_26.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_26.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_26.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_26.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_26.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_26.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_26.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_26.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_26.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_26.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_26.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_26.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_26.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_26.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_26.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_27.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_27.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_27.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_27.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_27.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_27.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_27.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_27.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_27.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_27.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_27.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_27.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_27.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_27.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_27.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_27.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_27.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_27.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_27.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_27.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_27.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_27.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_27.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_27.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_27.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_27.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_27.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_27.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_27.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_27.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_27.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_27.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_27.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_27.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_27.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_27.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_27.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_27.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_27.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_27.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_27.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_28.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_28.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_28.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_28.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_28.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_28.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_28.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_28.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_28.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_28.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_28.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_28.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_28.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_28.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_28.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_28.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_28.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_28.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_28.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_28.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_28.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_28.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_28.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_28.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_28.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_28.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_28.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_28.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_28.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_28.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_28.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_28.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_28.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_28.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_28.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_28.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_28.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_28.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_28.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_28.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_28.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;
defparam RGB_BRAM_29.INIT_00 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_01 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_02 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_03 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_04 = 'h00000000_00000001_01010000_01020405_090A0A09_04020000_00000000_00000000;
defparam RGB_BRAM_29.INIT_05 = 'h00000000_00000001_01010501_02040301_00000000_01000001_00000000_00020200;
defparam RGB_BRAM_29.INIT_06 = 'h00000000_00000000_0101060B_1D252F36_4846433E_221B120B_02000000_00000000;
defparam RGB_BRAM_29.INIT_07 = 'h00000000_01010000_00010503_00010000_00000000_00030100_020D1622_0B050000;
defparam RGB_BRAM_29.INIT_08 = 'h00000000_00000000_08101827_698298AB_C6C7C2B4_745A402E_0D050603_00010000;
defparam RGB_BRAM_29.INIT_09 = 'h00000000_01020000_041A3966_3B150002_00000000_00020000_1F4DA0CC_40140203;
defparam RGB_BRAM_29.INIT_0A = 'h00000000_00000001_111A2E44_A8C6DDF0_FDFEFDF6_B4946F50_160C0603_00010000;
defparam RGB_BRAM_29.INIT_0B = 'h00000000_00000001_09173962_3A190702_00000000_00030100_00010406_04020003;
defparam RGB_BRAM_29.INIT_0C = 'h00000000_00000100_0F152839_97B7D3E4_FCF6F3EC_A6836047_130C0504_00000001;
defparam RGB_BRAM_29.INIT_0D = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_0E = 'h00000000_01010000_0608101A_485C7383_99979186_53402F21_08020101_00000000;
defparam RGB_BRAM_29.INIT_0F = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_10 = 'h00000000_00000000_00000304_0E13181B_2425231F_0F0D0905_00000000_00000000;
defparam RGB_BRAM_29.INIT_11 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_12 = 'h00000000_00000000_00000101_01020303_03030303_02010100_00000000_00000000;
defparam RGB_BRAM_29.INIT_13 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_14 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_15 = 'h00000000_00000000_00000101_01010000_02030303_04040403_00000000_01010100;
defparam RGB_BRAM_29.INIT_16 = 'h00000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_17 = 'h00000000_00000000_00000102_0103080B_1A1E2123_1E1A1511_07040201_00000000;
defparam RGB_BRAM_29.INIT_18 = 'h00000103_03030000_00020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_19 = 'h00000000_00000000_00000003_11182433_60707A80_75665747_1C100B05_03010000;
defparam RGB_BRAM_29.INIT_1A = 'h01000100_457EAA9F_10040000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_1B = 'h00000000_00000000_00000308_1D2C4158_B1CFE4EE_D4BC9E7F_3322170E_02000001;
defparam RGB_BRAM_29.INIT_1C = 'h00000002_1E31413E_04020000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_1D = 'h00000000_00000000_00000304_1A273A4A_A0B7C8D3_B9A78C6F_2C1F100C_01010000;
defparam RGB_BRAM_29.INIT_1E = 'h00010101_00000001_01000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_1F = 'h00000000_00000000_00000101_0A0F1821_444E565A_50483C2F_110C0805_01010100;
defparam RGB_BRAM_29.INIT_20 = 'h00000000_00000000_00000000_00000000_01000000_01000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_21 = 'h00000000_00000000_00000000_02030304_0C0F1112_0E0D0C09_04030000_00000000;
defparam RGB_BRAM_29.INIT_22 = 'h00000000_00000000_00000000_01020203_05050505_02020100_01010000_00000000;
defparam RGB_BRAM_29.INIT_23 = 'h00000000_00000000_00000000_00000000_01010101_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_24 = 'h00000000_00000100_00010205_11131822_27262421_130E0A08_00010201_00000000;
defparam RGB_BRAM_29.INIT_25 = 'h00000000_00000000_00000000_00000000_01010000_00000000_00000000_01010101;
defparam RGB_BRAM_29.INIT_26 = 'h00000000_00000000_05070B13_3A495C6F_928E877B_44352419_06040201_00000000;
defparam RGB_BRAM_29.INIT_27 = 'h00000000_00000001_02030203_04050506_05050505_04040403_01010101_00000000;
defparam RGB_BRAM_29.INIT_28 = 'h00000000_00000000_080B1321_6581A3C1_F9F5E9D4_775C402C_0C040201_00000000;
defparam RGB_BRAM_29.INIT_29 = 'h00000001_03040505_090B090C_11131416_14141312_0E0E0D0B_07060806_02020100;
defparam RGB_BRAM_29.INIT_2A = 'h01000000_00010101_0608121D_526A869F_CCCCC2AF_654E3726_09030201_00000000;
defparam RGB_BRAM_29.INIT_2B = 'h01020305_07090C0E_171A1F21_2B2D3032_35353432_2724211E_15120F0D_06060503;
defparam RGB_BRAM_29.INIT_2C = 'h02010101_00000000_0000060B_202A353D_50504B43_241D140E_01000000_00000000;
defparam RGB_BRAM_29.INIT_2D = 'h0305070A_14171C22_373D4349_5C61676B_6D6B6A69_58534B45_2F2A2521_120E0B0A;
defparam RGB_BRAM_29.INIT_2E = 'h07050403_00000000_00000102_05070A0C_0D0D0C0A_07050302_00000000_00000000;
defparam RGB_BRAM_29.INIT_2F = 'h070A0D11_242C363F_606A7880_9EA5ACB0_B6B3B1AE_9991857B_574D423B_201A1410;
defparam RGB_BRAM_29.INIT_30 = 'h0C080503_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_31 = 'h0C12161B_3946525F_8D9CAAB5_D5DEE3E4_E8E8E7E2_CEC5BAAD_83716455_30262019;
defparam RGB_BRAM_29.INIT_32 = 'h100C0906_02010100_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_33 = 'h10171D25_47566576_AAB9C9D4_EFF6FAFA_FCFCFCF9_EDE3D9CC_9E8B7E6B_3F33291F;
defparam RGB_BRAM_29.INIT_34 = 'h110C0906_02010100_00000201_01010100_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_35 = 'h121A2028_4F5D6C7C_B3C3D3DE_F5FAFEFF_FEFFFFFE_F5EDE4D9_A8958572_44372D23;
defparam RGB_BRAM_29.INIT_36 = 'h0E0A0706_02010100_00000201_04050403_02020202_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_37 = 'h10161C23_45536271_A6B6C5D1_EDF3F8FA_FCFCFCF9_EAE1D7CA_9C8A7A68_3E322920;
defparam RGB_BRAM_29.INIT_38 = 'h0A060404_00000000_00000100_0F367EBD_501C0603_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_39 = 'h0C0F151C_343E4C59_8592A2AD_CDD6DCDD_E5E2DFDD_C7BEB0A4_796C5D52_2E261D17;
defparam RGB_BRAM_29.INIT_3A = 'h05040303_00000000_00000000_030C1B28_11070201_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_3B = 'h080A0D11_21273039_59616C76_90999FA1_A8A5A4A1_8B84796F_4E453B35_1D17110E;
defparam RGB_BRAM_29.INIT_3C = 'h02010000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_3D = 'h04050608_0F12161B_2F343C40_52575A5D_6262605D_4D48413C_2B261E1B_0F0C0806;
defparam RGB_BRAM_29.INIT_3E = 'h01000000_00000000_00000000_00000000_00000000_00000000_00000000_00000000;
defparam RGB_BRAM_29.INIT_3F = 'h00000103_05070A0C_14161A1D_25282B2D_2C2C2B28_22201D1A_12100E0B_06050301;



endmodule

